//
//  Ajustes.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Ajustes.h"
#import "iParkAppDelegate.h"
//#define _DEBUG2

@implementation Ajustes
@synthesize idioma = _idioma;
@synthesize intervalos = _intervalos;
@synthesize sePuedeLocalizar = _sePuedeLocalizar;


static Ajustes *_instancia = NULL;

- (id) init {
    self = [super init];
    if (self) {
    }
    return self;
}

+ (Ajustes *) instancia {
    // Si la instancia existe devolverla directamente
    if(_instancia)
        return _instancia;
    
    // Por cuestiones de rendimiento, solo se sincroniza la parte de crear el objeto
    @synchronized( self ) {
        if (_instancia == NULL)
            _instancia = [[self alloc] init];
    }
    return _instancia;
}

- (void)dealloc {
    [_idioma release];
    [_intervalos release];
    [super dealloc];
}

#pragma mark - Poner ajustes

- (void) ajustarIdiomaConInt: (int) idi {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setInteger: idi forKey: kKeyIdioma];
    [defaults synchronize];
}

- (void) ajustarIdioma: (Idioma *) idi {
    [self ajustarIdiomaConInt: idi.idioma];
}

- (void) ajustarIntervalosConPrimero: (int) i1 segundo: (int) i2 tercero: (int) i3 {
    NSArray *inter = [NSArray arrayWithObjects:
                      [NSNumber numberWithInt: i1] ,
                      [NSNumber numberWithInt: i2] ,
                      [NSNumber numberWithInt: i3] ,
                      nil];
    [self ajustarIntervalos: inter];
}

- (void) ajustarIntervalos: (NSArray *) intervalos {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject: intervalos forKey: kKeyIntervalos];
    [defaults synchronize];
}

- (void) ajustarSePuedeLocalizar: (BOOL) localizar {
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setBool: localizar forKey:kKeyLocalizar];
    [defaults synchronize];
}

#pragma mark - sync

- (void) cambiosEnPreferendias {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults synchronize];
    // Guardar el idioma antiguo
    int idiomaAntiguo = self.idioma.idioma;
    // Obtener el nuevo idioma
    self.idioma = [Idioma idiomaConInt: [defaults integerForKey: kKeyIdioma]];
    // Si el idioma ha cambiado, cambiar los títulos de las pestañas
    if (idiomaAntiguo != self.idioma.idioma) {
        UITabBarController *tabBarController = (UITabBarController *) 
                    [[UIApplication sharedApplication] keyWindow].rootViewController;
        [[tabBarController.tabBar.items objectAtIndex: kIndexCalcular] 
                    setTitle: [TraductorHash traducir: @"Calcular"]];
        [[tabBarController.tabBar.items objectAtIndex: kIndexSectores] 
                    setTitle: [TraductorHash traducir: @"Sectores"]];
        [[tabBarController.tabBar.items objectAtIndex: kIndexInfo]     
                    setTitle: [TraductorHash traducir: @"Info"]];
        [[tabBarController.tabBar.items objectAtIndex: kIndexAlarmas]  
                    setTitle: [TraductorHash traducir: @"Alarmas"]];
        [[tabBarController.tabBar.items objectAtIndex: kIndexAjustes]  
                    setTitle: [TraductorHash traducir: @"Ajustes"]];
    }
    
    self.intervalos = (NSArray *) [defaults objectForKey: kKeyIntervalos];
    self.sePuedeLocalizar = [defaults boolForKey: kKeyLocalizar];
}

# pragma mark - valores disponibles

- (NSArray *) idiomasDisponibles {
    // Obtener el path para el diccionario de settings en "Settings.bundle"
    NSString *pathDicSettings = [[[NSBundle mainBundle] pathForResource:@"Settings" 
                                                                 ofType:@"bundle"] 
                                         stringByAppendingPathComponent:@"Root.plist"];
    // Obtener el diccionario en sí
    NSDictionary *dicSettings = [NSDictionary dictionaryWithContentsOfFile: pathDicSettings];
    // Obtener el array de grupos de preferencias
    NSArray *arrayGrupos = [dicSettings objectForKey: @"PreferenceSpecifiers"];
    // Obtener el diccionario correspondiente al grupo que queremos: el de idiomas
    NSDictionary *dicGrupo = [arrayGrupos objectAtIndex: kIndiceIdiomas];
    // Obtener el array de valores
    NSArray *arrayIdiomasInt = [dicGrupo objectForKey: @"Values"];
    // Crear un array de idiomas a partir del array de intes
    NSMutableArray *res = [NSMutableArray arrayWithCapacity: [arrayIdiomasInt count]];
    for (NSNumber *n in arrayIdiomasInt)
        [res addObject: [Idioma idiomaConInt: [n intValue]]];
    
    #ifdef _DEBUG2
        NSLog(@"__%@__\n%@\n%@\n%@\n%@\n%@" , NSStringFromSelector( _cmd), dicSettings , arrayGrupos , dicGrupo , arrayIdiomasInt , res);
    #endif
    return res;
}
- (NSArray *) intervalosDisponibles {
    NSString *pathDicSettings = [[[NSBundle mainBundle] pathForResource:@"Settings" 
                                                                 ofType:@"bundle"] 
                                 stringByAppendingPathComponent:@"Root.plist"];
    NSDictionary *dicSettings = [NSDictionary dictionaryWithContentsOfFile: pathDicSettings];
    NSArray *arrayGrupos = [dicSettings objectForKey: @"PreferenceSpecifiers"];
    NSDictionary *dicGrupo = [arrayGrupos objectAtIndex: kIndiceIntervalos];
    return [dicGrupo objectForKey: @"Values"];
}

@end
