//
//  AjustesTableViewController.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "AjustesTableViewController.h"
//#define _DEBUG2

@implementation AjustesTableViewController
@synthesize cellIdioma = _cellIdioma;
@synthesize cellLocalizar = _cellLocalizar;
@synthesize cellIntervalos = _cellIntervalos;

- (id)initWithStyle: (UITableViewStyle)style {
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc {
    [super dealloc];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated {
    #ifdef _DEBUG2
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    [super viewWillAppear:animated];
    // Actualizar los cambios en las preferencias
    [[Ajustes instancia] cambiosEnPreferendias];
    
    // Que se nos notifique cuando la apliación pasa a activo
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(appDidBecomeActive) 
                                                 name: UIApplicationDidBecomeActiveNotification
                                               object: nil];
    
    //Poner el título a la navigation Bar
    self.navigationItem.title = [TraductorHash traducir:@"Ajustes"];
    
    // Recargar los datos por las traducciones
    [self.tableView reloadData];
    
    // Cambiar el switch si procede
    UISwitch *sw = (UISwitch *) self.cellLocalizar.accessoryView;
    [sw setOn: [Ajustes instancia].sePuedeLocalizar animated: animated];
}

- (void) appDidBecomeActive {
    [self viewWillAppear:NO];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver: self];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    int res;
    if (section == 0 )
        res = 1;
    else 
        res = 2;
    return res;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    // 0,0 idioma
    if (indexPath.section == 0 && indexPath.row == 0) {
        // Si la cell de idioma no ha sido instanciada:
        if (self.cellIdioma == nil) {
            self.cellIdioma = [[[UITableViewCell alloc] 
                                    initWithStyle: UITableViewCellStyleValue1 
                                  reuseIdentifier: @"idioma"] autorelease];
            // Configurar los datos que no van a cambiar
            self.cellIdioma.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        }
        // Ponerle el título y subtítulo correspondientes
        self.cellIdioma.textLabel.text = [TraductorHash traducir:@"Idioma"];
        self.cellIdioma.detailTextLabel.text = [[Ajustes instancia].idioma description];
        
        return self.cellIdioma;
    }
    // 1,0 intervalos
    else if (indexPath.section == 1 && indexPath.row == 0) {
        // Instanciar si no existe
        if (self.cellIntervalos == nil) {
            self.cellIntervalos = [[[UITableViewCell alloc] 
                                    initWithStyle: UITableViewCellStyleValue1 
                                  reuseIdentifier: @"intervalos"] autorelease];
            
            self.cellIntervalos.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        }
        // Poner título
        self.cellIntervalos.textLabel.text = [TraductorHash traducir:@"Avisos previos"];
        // Poner subtíulo, con formato
        NSString *formato = [TraductorHash traducir:@"%d, %d o %d minutos antes"];
        // y valores:
        NSArray *inter = [Ajustes instancia].intervalos;
        int i1 = [(NSNumber *) [inter objectAtIndex: 0] intValue];
        int i2 = [(NSNumber *) [inter objectAtIndex: 1] intValue];
        int i3 = [(NSNumber *) [inter objectAtIndex: 2] intValue];
        self.cellIntervalos.detailTextLabel.text = [NSString stringWithFormat: formato , i1 , i2 , i3];
        
        return self.cellIntervalos;
    }
    // 1,1 localizar
    else {
        // Si no existe crearla
        if (self.cellLocalizar == nil) {
            self.cellLocalizar = [[[UITableViewCell alloc] 
                                        initWithStyle: UITableViewCellStyleValue1 
                                      reuseIdentifier: @"localizar"] autorelease];
            // Crear el switch 
            UISwitch *sw = [[UISwitch alloc] init];
            // Especificar el método al que llamara
            [sw addTarget: self  action: @selector(cambiaLocalizar:) 
                       forControlEvents: UIControlEventValueChanged];
            // Cambiarle el estado según los ajustes
            [sw setOn: [Ajustes instancia].sePuedeLocalizar animated: NO];
            // Añadirlo como vista accesoria
            self.cellLocalizar.accessoryView = sw;
            [sw release];
            // No seleccionable
            self.cellLocalizar.selectionStyle = UITableViewCellEditingStyleNone;
        }
        self.cellLocalizar.textLabel.text = [TraductorHash traducir:@"Permitir localizar"];
        return self.cellLocalizar;
    }
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Cuando se seleccione una fila debemos rellenar:
    //  - Array de valores con su correspondiente array de título
    NSMutableArray *arrayTitulos;
    NSArray *arrayValores;
    //  - El índice de la tabla que estará seleccinado al cargarse
    int indiceASeleccionar = 0;
    //  - La acción que se debe ejecutar cuando se seleccione un nuevo valor
    SEL accion;
    //  - El título de la navigation controller
    NSString *tituloController;
    //  - El texto que se añade como footer
    NSString *explicacion = nil;
    // Idioma
    if (indexPath.section == 0 && indexPath.row == 0) {
        tituloController = @"Idioma";
        // La acción que ejecutara sobre la instancia de ajustes, ajustarIdioma:
        accion = @selector(ajustarIdioma:);
        
        // Obtenemos el array de valores disponibles
        arrayValores = [[Ajustes instancia] idiomasDisponibles];
        #ifdef _DEBUG
            NSLog(@"__%@__.%@." , NSStringFromSelector( _cmd) , arrayValores);
        #endif
        
        // Y vamos rellenando el array de títulos para cada valor
        arrayTitulos = [NSMutableArray arrayWithCapacity: [arrayValores count]];
        int idiomaActual = [Ajustes instancia].idioma.idioma;
        int i = 0;
        // Para cada idioma:
        for (Idioma *idi in arrayValores) {
            // Guardarnos como índice el correspondiente al idioma actual
            if (idi.idioma == idiomaActual ) 
                indiceASeleccionar = i;
            // Añadir el idioma
            [arrayTitulos addObject: [idi description]];
            i++;
        }
    }
    // Intervalos, avisos previos
    else if (indexPath.section == 1 && indexPath.row == 0) {
        tituloController = @"Avisos previos";
        // La acción que ejecutara sobre la instancia de ajustes, ajustarIntervalos:
        accion = @selector(ajustarIntervalos:);
        // Este ajuste requiere explicación
        explicacion = @"Seleccione los valores que estarán disponibles al añadir nuevas alarmas.";
        
        // Obtenemos el array de valores disponibles
        arrayValores = [[Ajustes instancia] intervalosDisponibles];
        
        // Y vamos rellenando el array de títulos para cada valor
        arrayTitulos = [NSMutableArray arrayWithCapacity: [arrayValores count]];
        // Formato para los títulos
        NSString *formato = [TraductorHash traducir:@"%d, %d o %d minutos antes"];
        NSArray *interValoActual = [Ajustes instancia].intervalos;
        int i = 0;
        // Para cada intervalo:
        for (NSArray *inter in arrayValores) {
            // Guardarnos el indice del intervalo que querremos seleccionar
            if ([inter isEqual: interValoActual]) 
                indiceASeleccionar = i;
            // Obtener los números
            int i1 = [(NSNumber *) [inter objectAtIndex:0] intValue];
            int i2 = [(NSNumber *) [inter objectAtIndex:1] intValue];
            int i3 = [(NSNumber *) [inter objectAtIndex:2] intValue];
            [arrayTitulos addObject: [NSString stringWithFormat: formato , i1 , i2 , i3]];
            i++;
        }
    }
    // Localizable
    else {
        // No hace nada, volver
        return;
    }
    
    // Si llegamos hasta aquí, creamos el controlador que presentará la lista de opciones a seleccionar
    AjustesSeleccionarViewController *controllerOpciones = [[AjustesSeleccionarViewController alloc] 
                                                            initWithStyle: UITableViewStyleGrouped];
    // Rellenamos los atributos que hemos configurado antes
    controllerOpciones.titulos = arrayTitulos;
    controllerOpciones.valores = arrayValores; 
    // El índice a seleccionar se crea como un index path con sección 0 y fila=indice
    controllerOpciones.ultimoIndexPath = [NSIndexPath indexPathForRow: indiceASeleccionar inSection:0];
    // El selector se le hará a la instancia del singleton de ajustes
    controllerOpciones.target = [Ajustes instancia];
    controllerOpciones.action = accion;
    // Necesita el título y explicación originarios en castellano para poder traducirlos cada vez
    controllerOpciones.titulo = tituloController;
    controllerOpciones.explicacion = explicacion;
    // Navegar al controlador creado
    [self.navigationController pushViewController: controllerOpciones animated:YES];
    [controllerOpciones release];
}

#pragma mark - Acciones

- (IBAction) cambiaLocalizar: (id) sender {
    UISwitch *sw = (UISwitch *) sender;
    [[Ajustes instancia] ajustarSePuedeLocalizar: sw.on];
}



@end
