//
//  AlarmasTableViewController.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "AlarmasTableViewController.h"
//#define _DEBUG3

@implementation AlarmasTableViewController
@synthesize imagenQuitar = _imagenQuitar;

- (id)initWithStyle:(UITableViewStyle)style {
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc {
    [_imagenQuitar release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}


#pragma mark - View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Poner el alto de las celdas
    self.tableView.rowHeight = kAltoFila;
    
    // Crear un botón derecho
    UIBarButtonItem *editButton = [[UIBarButtonItem alloc] 
                                   initWithTitle: [TraductorHash traducir:@"Hecho"]
                                           style: UIBarButtonItemStyleBordered 
                                          target: self 
                                          action: @selector(conmutarEdit)];
    self.imagenQuitar = [UIImage imageNamed: kImagenQuitar];
    editButton.image = self.imagenQuitar;
    // añadir el botón a la derecha
    self.navigationItem.rightBarButtonItem = editButton;
    [editButton release];
    
    // Añadir la imagen de fondo
    UIImageView *iv = [[UIImageView alloc] initWithImage: [UIImage imageNamed: kImagenFondoTable]];
    self.tableView.backgroundView = iv;
    [iv release];
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    [self setImagenQuitar: nil];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // Ver si ha habido cambios en las preferencias
    [[Ajustes instancia] cambiosEnPreferendias];
    
    // Que se nos notifique al become active por si se han pasado alarmas
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(appDidBecomeActive) 
                                                 name: UIApplicationDidBecomeActiveNotification
                                               object: nil];
    
    
    //Poner el título a la navigation Bar
    self.navigationItem.title = [TraductorHash traducir:@"Alarmas"];
    // Actualizar el título de los botones
    if (self.editing) {
        self.navigationItem.rightBarButtonItem.title = [TraductorHash traducir:@"Hecho"];
        self.navigationItem.leftBarButtonItem.title  = [TraductorHash traducir:@"Todas"];
    }
    
    // Recargar la tabla por si se han pasado alarmas
    [self.tableView reloadData];
}

- (void) appDidBecomeActive {
    [self viewWillAppear:NO];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // Si navegamos fuera de esta vista, no se nos notifica.
    [[NSNotificationCenter defaultCenter] removeObserver: self];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    int n = [[[UIApplication sharedApplication] scheduledLocalNotifications] count];
    // Siempre hay al menos una fila en la tabla, la de información
    return (n > 0) ? n : 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"AlarmasTableViewControllerCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle: UITableViewCellStyleSubtitle 
                                       reuseIdentifier: CellIdentifier] autorelease];
        
        // Configurar los elementos que no cambian de la cell
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.textLabel.font = [UIFont boldSystemFontOfSize: kTamFuenteCell];
        cell.textLabel.lineBreakMode = UILineBreakModeHeadTruncation;
        cell.detailTextLabel.font = [UIFont systemFontOfSize: kTamFuenteCell];
        cell.detailTextLabel.lineBreakMode = UILineBreakModeHeadTruncation;
        
        // Añadir la imagen de fondo
        UIImageView *iv = [[UIImageView alloc] initWithImage: [UIImage imageNamed: kImagenCell]];
        cell.backgroundView = iv;
        [iv release];
        // Añadir la imagen de fondo para cuando se seleccione la cell
        UIImageView *ivSelec = [[UIImageView alloc] initWithImage: [UIImage imageNamed: kImagenCellSelec]];
        cell.selectedBackgroundView = ivSelec;
        [ivSelec release];
        
        // Hacer que las dos etiquetas sean transparentes
        cell.detailTextLabel.backgroundColor = 
        cell.textLabel.backgroundColor = [UIColor clearColor];
    }
    
    // Obtener el array de notificaciones y comprobar que no está vacío
    NSArray *notificationArray = [[UIApplication sharedApplication] scheduledLocalNotifications];
    // Si es mayor que 0, los cell tienen la información de cada alarma
    if ([notificationArray count] > 0 ) {
        // Obtener la notificación correspondiente a la fila
        UILocalNotification *notif = [notificationArray objectAtIndex:indexPath.row];
        
        // Formatear la fecha de la alarma
        NSDateFormatter *df = [[NSDateFormatter alloc] init];
        [df setDateFormat: @"dd/MM HH:mm"];
        NSString *text = [NSString stringWithFormat:@"%@: %@" , 
                          [TraductorHash traducir:@"Alarma"] ,
                          [df stringFromDate: notif.fireDate]];
        cell.textLabel.text =  text;
        
        // Formatear la fecha del fin de estacionamiento
        [df setDateFormat: @"HH:mm"];
        NSString *horaFin = [df stringFromDate: [notif.userInfo objectForKey: @"horaFin"]];
        NSString *detailedText = [NSString stringWithFormat: @"%@: %@",
                                  [TraductorHash traducir:@"Fin de estacionamiento"] ,
                                   horaFin];
        cell.detailTextLabel.text = detailedText;
        
        // Quitar la imagen por si tenía
        cell.imageView.image = nil;
        [df release];
    }
    // Si no hay notificaciones, mostramos una cell con un mensaje y una imagen
    else {
        #ifdef _DEBUG
            NSLog(@"__%@__vacio" , NSStringFromSelector( _cmd));
        #endif
        cell.textLabel.text = [TraductorHash traducir:@"Añadir alarmas en 'Calcular'"];
        // Quitar el subtítulo
        cell.detailTextLabel.text = nil;
        cell.imageView.image = [UIImage imageNamed: @"calcular"];
    }
    return cell;
}


// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    int numNotif = [[[UIApplication sharedApplication] scheduledLocalNotifications] count];
    // Se puede editar siempre excepto si no hay notificaciones Y es la primera fila.
    // O lo que es lo mismo, si es una fila distinta a la primera o hay alguna notificación, se
    // puede editar.
    return !( ( numNotif == 0 ) && ( indexPath.row == 0 ) );
}

// Método para devolver el título del botón rojo de borrar traducido.
- (NSString *)tableView:(UITableView *)tableView titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [TraductorHash traducir:@"Eliminar"];
}

// Por si ha entrado en modo editar una sola fila (con el gesto
// de deslizar un dedo) al salirnos, debemos quitarle el título ya que 
// en algún idioma aparece el título junto a la imagen
- (void)tableView:(UITableView *)tableView didEndEditingRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.editing) {
        [self conmutarEdit];
    }
    #ifdef _DEBUG3
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
}




// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    // Si estamos borrando:
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        NSArray *notificationArray = [[UIApplication sharedApplication] scheduledLocalNotifications];
        // Comprobar que no nos salimos de los límites
        if ( [notificationArray count] >= indexPath.row + 1 ) {
            #ifdef _DEBUG
                NSLog(@"__%@__" , NSStringFromSelector( _cmd));
            #endif
            // Obtener la notificación
            UILocalNotification *notif = [notificationArray objectAtIndex: indexPath.row];
            // Cancelarla
            [[UIApplication sharedApplication] cancelLocalNotification: notif];
            
            // Empiezo a editar
            [self.tableView beginUpdates];
            // Quito la fila que se debe quitar
            [self.tableView deleteRowsAtIndexPaths: [NSArray arrayWithObject: indexPath] 
                                  withRowAnimation: UITableViewRowAnimationRight];
            int numNotifActuales = [[[UIApplication sharedApplication] scheduledLocalNotifications] count];
            // Pero si el numero de notificaciones ya es 0, entonces añado una cell, la de la información
            if ( numNotifActuales == 0) {
                [self.tableView insertRowsAtIndexPaths: [NSArray arrayWithObject: indexPath] 
                                      withRowAnimation: UITableViewRowAnimationLeft];
            }
            [self.tableView endUpdates];
            // termino de editar
        }

    }
}

#pragma mark - Metodos en respuesta a botones

// Función que conmuta entre editando o no editando
// Se llama al tocar el botón superior derecho
- (void) conmutarEdit {
    [self setEditing:!self.editing animated:YES];

    if ( self.editing ) {
        // Cambiar estilo y quitar la imagen al botón derecho
        self.navigationItem.rightBarButtonItem.style = UIBarButtonItemStyleDone;
        self.navigationItem.rightBarButtonItem.title = [TraductorHash traducir:@"Hecho"];
        self.navigationItem.rightBarButtonItem.image = nil;
        
        // Añadir el botón izquierdo
        UIBarButtonItem *borraTodasButton = [[UIBarButtonItem alloc] 
                                             initWithTitle: [TraductorHash traducir:@"Todas"]
                                             style: UIBarButtonItemStyleBordered 
                                             target: self 
                                             action: @selector(borraTodas)];
        self.navigationItem.leftBarButtonItem = borraTodasButton;
        [borraTodasButton release];
        
    }
    else {
        // Cambiar el estilo y poner la imagen al botón derecho
        self.navigationItem.rightBarButtonItem.style = UIBarButtonItemStyleBordered;
        self.navigationItem.rightBarButtonItem.title = nil;
        self.navigationItem.rightBarButtonItem.image = self.imagenQuitar;
        
        
        // Quitar el botón izquierdo
        self.navigationItem.leftBarButtonItem = nil;
    }
}

// Método que hace oscilar horizontalmente una view en un intervalo de tiempo con 
// una amplitud medida en píxeles
- (void) agitaView: (UIView *) v conDuracion: (NSTimeInterval) dur amplitud: (int) a {
    // El movimiento tiene tres fases, la duración la dividimos en 3
    dur /= 3;
    
    /** Mover hacia la derecha a píxeles **/
    [UIView animateWithDuration: dur
                          delay: 0
                        options: (UIViewAnimationOptionAllowUserInteraction |
                                  UIViewAnimationOptionBeginFromCurrentState )
                     animations:^ {
                         
                         CGPoint c1 = v.center;
                         c1.x += a;
                         v.center = c1;
                     } 
                     completion:^(BOOL finished){
                         /** Al acabar, mover hacia la izquierda 2a píxeles **/
                         [UIView animateWithDuration: dur
                                               delay: 0
                                             options: (UIViewAnimationOptionAllowUserInteraction |
                                                       UIViewAnimationOptionBeginFromCurrentState )
                                          animations:^ {
                                              
                                              CGPoint c1 = v.center;
                                              c1.x -= 2*a;
                                              v.center = c1;
                                          } 
                                          completion:^(BOOL finished){
                                              /** Al acabar, mover hacia la izquierda a píxeles para volver a la posición inicial **/       
                                              [UIView animateWithDuration: dur
                                                                    delay: 0
                                                                  options: (UIViewAnimationOptionAllowUserInteraction |
                                                                            UIViewAnimationOptionBeginFromCurrentState )
                                                               animations:^ {
                                                                   
                                                                   CGPoint c1 = v.center;
                                                                   c1.x += a;
                                                                   v.center = c1;
                                                               } 
                                                               completion:^(BOOL finished){
                                                               } ];
                                          } ];
                     } ];
}

// Función que se llama al tocar el botón izquierdo (si está visible)
- (void) borraTodas {
    if ( [[[UIApplication sharedApplication] scheduledLocalNotifications] count] > 0 ) {
        // Quitar todas las alarmas
        [[UIApplication sharedApplication] cancelAllLocalNotifications];
        // Recargar la tabla
        [self.tableView beginUpdates];
        [self.tableView deleteSections: [NSIndexSet indexSetWithIndex:0] 
                      withRowAnimation: UITableViewRowAnimationRight];
        [self.tableView insertSections: [NSIndexSet indexSetWithIndex:0]
                      withRowAnimation: UITableViewRowAnimationLeft];
        [self.tableView endUpdates];
    }
    else {
        // Ofrecer feedback, que se sepa que borrar todas cuando no hay filas no hace nada
        UITableViewCell *cellPonerAlarmas = [[self.tableView visibleCells] objectAtIndex:0];
        [self agitaView:cellPonerAlarmas conDuracion:0.3 amplitud: 20];
    }
}

@end
