//
//  Calculadora.h
//  iPark
// 
//  Clase que engloba todos los datos que utiliza la pestaña calcular
//  (Fecha, euros...).
//
//  También se encarga de hacer los cálculos de la zona azul
//  
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "Zona.h"
#import "Fecha.h"
#import "Periodo.h"
#import "Euros.h"

// El número del día correspondiente al domingo y al sábado
#define kDomingo            1
#define kSabado             7
#define kViernes            6

// Constantes que representan una hora en minutos.
// 
// Por ejemplo, k0830 representa las 8 horas
// y 30 minutos de la mañana, eso se traduce en
// 8*60+30 = 510 minutos
#define k0830               510 // 8*60+30
#define k1400               840 //14*60+00
#define k1600               960 //16*60+00
#define k2000              1200 //20*60+00
#define k0830Siguiente     1950 //24*60+8*60+30
#define k0830Siguiente2    3390 //48*60+8*60+30
#define k0200               120 // 2*60+30
#define k1230               750 //12*60+30
#define k4230              2550 //42*60+30
#define k2400              1440 //24*60+00

@interface Calculadora : NSObject {
    Zona *_zona;
    Fecha *_fechaInicio;
    Periodo *_periodo;
    Euros *_euros;
    
}
// Datos que engloba la calculadora:
@property (nonatomic , retain ) Zona *zona;
@property (nonatomic , retain ) Fecha *fechaInicio;
@property (nonatomic , retain ) Periodo *periodo;
@property (nonatomic , retain ) Euros *euros;

// Constructor por defecto con todo a nil
- (id)init;
// Constructor con todos los datos
- (id)initWithZona: (Zona *) z fechaInicio: (Fecha *) fi periodo: (Periodo *) p euros: (Euros *) e;
// Devuelve la representación como cadena de texto de la calculadora
- (NSString *) description;

// Actualiza el periodo a partir de los demás datos y devuelve el resultado
- (Periodo *) calcularPeriodo;
// Actualiza los euros a partir de los demás datos y devuelve el resultado
- (Euros *) calcularEuros;
// Calcula, a partir de la fecha inicial y con el periodo, la fecha hasta la que 
// se puede aparcar.
- (NSDate *) calcularFechaFinal;
// Calcula, a partir de una fecha dada y el periodo, la fecha hasta la que se
// puede aparcar.
- (NSDate *) calcularFechaFinalDesdeFecha: (NSDate *) f;


@end
