//
//  Calculadora.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Calculadora.h"
//#define _DEBUG

@implementation Calculadora
@synthesize zona = _zona;
@synthesize fechaInicio = _fechaInicio;
@synthesize periodo = _periodo;
@synthesize euros = _euros;

#pragma mark - Constructores, limpiadores, ...

// Constructor por defecto con todo a nil
- (id)init {
    self = [super init];
    if (self) {
        self.zona = nil;
        self.fechaInicio = nil;
        self.periodo = nil;
        self.euros = nil;
    }
    return self;
}

// Constructor con todos los datos
- (id)initWithZona: (Zona *) z fechaInicio: (Fecha *) fi periodo: (Periodo *) p euros: (Euros *) e {
    self = [self init];
    if (self) {
        self.zona = z;
        self.fechaInicio = fi;
        self.periodo = p;
        self.euros = e;
    }
    return self;
}

- (NSString *) description {
    return [NSString stringWithFormat:@"\n%@\n%@\n%@\n%@\n" , self.zona , self.fechaInicio , self.periodo , self.euros];
}

- (void)dealloc {
    [_zona release];
    [_fechaInicio release];
    [_periodo release];
    [_euros release];
    [super dealloc];
}


#pragma mark - Getters calculadores
// Actualiza el periodo a partir de los demás datos y devuelve el resultado
- (Periodo *) calcularPeriodo {
    // Utiliza los cálculos concretos con la tarifa de cada zona
    self.periodo = [self.zona calcularConEuros: self.euros];
    return self.periodo;
}
// Actualiza los euros a partir de los demás datos y devuelve el resultado
- (Euros *) calcularEuros {
    self.euros = [self.zona calcularConPeriodo: self.periodo];
    return self.euros;
}


// Método auxiliar para calcularFechaFinal
// Dada una fecha, si cae en un periodo en el que no hay zona
// azul, lo avanza hasta el final de este.
- (NSDate *) avanzar: (NSDate *) fecha {
    #ifdef _DEBUG
        NSDateFormatter *df = [[NSDateFormatter alloc] init];
        [df setDateFormat: @"dd/MM HH:mm"];
        NSString *st = [df stringFromDate: fecha];
        NSLog(@"__%@__%@" , NSStringFromSelector( _cmd) , st);
        [df release];
    #endif
    
    // Obtener el día de la semana y la hora
    NSCalendar *calendar = [NSCalendar currentCalendar];
    NSDateComponents *components = [calendar components:(kCFCalendarUnitWeekday | kCFCalendarUnitHour | kCFCalendarUnitMinute) fromDate: fecha];
    int diaSemana = [components weekday];
    // Vamos a trabajar con la hora convertida a minutos.
    int minsParaComparar = [components hour]*60 + [components minute];
    
    // La diferencia que se le sumará al final a la fecha, en principio, 0.
    int diferencia = 0;
    
    /****** DOMINGO *******/
    if ( diaSemana == kDomingo ) {
        // Si es domingo avanzamos hasta las 08:30 del día siguiente
        diferencia = k0830Siguiente - minsParaComparar;
    }
    /****** DOMINGO *******/
    else if ( diaSemana == kSabado ) {
        // Antes de las 08:30
        if      (                                 minsParaComparar < k0830 ) {
            // Avanzamos hasta las 08:30
            diferencia = k0830 - minsParaComparar;
        }
        // Entre las 08:30 y las 14:00 HAY zona azul
        else if ( (k0830 <= minsParaComparar) && (minsParaComparar < k1400) ) {
            // No avanzamos
        }
        // A partir de las 14:00 no hay zona azul
        else if (  k1400 <= minsParaComparar ) {
            // Avanzamos hasta las 08:30 del lunes!
            diferencia = k0830Siguiente2 - minsParaComparar;
        }
        #ifdef _DEBUG
                NSLog(@"__%@__sabadodif h: %d m:%d" , NSStringFromSelector( _cmd) , diferencia /60 , diferencia%60);
        #endif
    }
    /******* ENTRE SEMANA *******/
    else {
        // Antes de las 08:30
        if      (                                 minsParaComparar < k0830  ) {
            // Avanzamos hasta las 08:30
            diferencia = k0830 - minsParaComparar;
        }
        // De 08:30 a 14:00 HAY zona azul
        else if ( (k0830 <= minsParaComparar) && (minsParaComparar < k1400) ) {
            // No avanzamos
        }
        // De 14:00 a 16:00 no hay zona azul
        else if ( (k1400 <= minsParaComparar) && (minsParaComparar < k1600) ) {
            // Avanzamos hasta las 16:00
            diferencia = k1600 - minsParaComparar;
        }
        // De 16:00 a 20:00 HAY zona azul
        else if ( (k1600 <= minsParaComparar) && (minsParaComparar < k2000) ) {
            // No avanzamos
        }
        // A partir de las 20:00, no hay zona azul
        else if (  k2000 <= minsParaComparar ) {
            diferencia = k0830Siguiente - minsParaComparar;
        }
    }
    #ifdef _DEBUG
        NSDateFormatter *df2 = [[NSDateFormatter alloc] init];
        [df2 setDateFormat: @"dd/MM HH:mm"];
        NSString *st2 = [df2 stringFromDate: [fecha dateByAddingTimeInterval: diferencia*60]];
        NSLog(@"_____%@__%@" , NSStringFromSelector( _cmd) , st2);
        [df2 release];
    #endif
    // Le sumamos la diferencia (que la tenemos en minutos y la necesitamos en segundos)
    // Devolvemos sin modificar la fecha que nos pasan
    return [fecha dateByAddingTimeInterval: diferencia*60];
}

// Método más concreto, llamará a fechaFinalDesdeFecha
// Calcula, a partir de la fecha inicial y con el periodo, la fecha hasta la que 
// se puede aparcar.
- (NSDate *) calcularFechaFinal {
    return [self calcularFechaFinalDesdeFecha: [self.fechaInicio date]];
}


// Método más general:
// Calcula, a partir de una fecha dada y el periodo, la fecha hasta la que se
// puede aparcar.
- (NSDate *) calcularFechaFinalDesdeFecha: (NSDate *) f {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    
    // Avanzamos la fecha hasta que esté en un periodo con zona azul
    NSDate *fechaRes = [self avanzar: f];
 
    // Ahora tenemos que fecha está en el intervalo:
    // (1)  [08:30 .. 13:59]
    //  ó
    // (2)  [16:00 .. 19:59]
    #ifdef _DEBUG
        NSDateFormatter *df2 = [[NSDateFormatter alloc] init];
        [df2 setDateFormat: @"dd/MM HH:mm"];
        NSString *st2 = [df2 stringFromDate: fechaRes];
        NSLog(@"_____%@__%@" , NSStringFromSelector( _cmd) , st2);
        [df2 release];
    #endif
    
    // Obtenemos los componentes necesario (hora, minuto, dia de la semana)
    NSCalendar *calendar = [NSCalendar currentCalendar];
    NSDateComponents *components = [calendar components:( kCFCalendarUnitWeekday |kCFCalendarUnitHour | kCFCalendarUnitMinute) fromDate: fechaRes];
    
    int diaSemana = [components weekday];
    // Hora de comienzo, representada en minutos
    int minsPrincipio = [components hour]*60 + [components minute];
    // Le sumamos el periodo de estacionamiento 
    int minsFinal = minsPrincipio + self.periodo.horas*60 + self.periodo.minutos;
    
    #ifdef _DEBUG
        NSLog(@"__%@__ minsPrincipio %02d:%02d minsFinal %02d:%02d" , NSStringFromSelector( _cmd), minsPrincipio /60 , minsPrincipio%60 , minsFinal/60 , minsFinal%60);
    #endif
    
    /***** Intervalo (1) *****/
    if ( (k0830 <= minsPrincipio) && (minsPrincipio < k1400) ) {
        #ifdef _DEBUG
                NSLog(@"__%@__ intervalo1" , NSStringFromSelector( _cmd));
        #endif

        if ( diaSemana == kSabado ) {
            #ifdef _DEBUG
                        NSLog(@"__%@__ SABADO" , NSStringFromSelector( _cmd));
            #endif
            // Si sabado y:
            // cae antes de las 14:00 --> nada
            // Si cae despúes --> +42h30 min (hasta el lunes saltándonos el domingo)
            if (k1400 <= minsFinal)
                minsFinal += k4230;
        }
        else {
            #ifdef _DEBUG
                        NSLog(@"__%@__ ENTRESEMANA" , NSStringFromSelector( _cmd));
            #endif
            // Si entre semana y:
            //   cae antes de las 14:00    --> no hacemos nada
            //   cae despúes --> +2h
            if (k1400 <= minsFinal)
                minsFinal += k0200;
            
            // Ahora, si ahora caemos entre las 16:00 y las 20:00, nada
            // si caemos a las 20:00 o más, +12h30min
            if (  k2000 <= minsFinal )
                minsFinal += k1230;
        }
    }
    /***** Intervalo (2) *****/
    else { //if ( (k1600 <= minsPrincipio) && (minsPrincipio < k2000) ) {
        #ifdef _DEBUG
                NSLog(@"__%@__ intervalo2" , NSStringFromSelector( _cmd));
        #endif
        // Intervalo (2): (sólo entre semana)
        //   cae antes de 20:00 --> nada
        //   cae después  --> +12h30
        if (  k2000 <= minsFinal )
            minsFinal += k1230;
        
        // Si nos hemos pasado de las 14:00 del día siguiente
        // Si era de lunes a jueves 
        //    +2 horas (del periodo 14:00-16:00)
        // Si era viernes, pasamos al sábado y
        //    y +42:30 hasta el lunes
        if ( k1400 <= (minsFinal - k2400) ) {
            #ifdef _DEBUG
                        NSLog(@"__%@__ dia %d" , NSStringFromSelector( _cmd) , diaSemana);
            #endif
            if (diaSemana == kViernes)
                minsFinal += k4230;
            else 
                minsFinal += k0200;
        }
        
    }
    
    // Añadimos a la fecha los minutos de más
    fechaRes = [fechaRes dateByAddingTimeInterval: (minsFinal-minsPrincipio)*60];
    
    // Si hemos caido otra vez en un periodo que no tiene zona azul, lo avanzamos hasta el final
    return fechaRes;
}

@end
