//
//  CalcularTableViewController.h
//  iPark
//
//  Clase encargada de gestionar la primera pestaña: calcular
//  
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "TraductorHash.h"
#import "FechaActionSheet.h"
#import "EurosActionSheet.h"
#import "TiempoActionSheet.h"
#import "Calculadora.h"
#import "ZonaAzul.h"
#import "CustomSegmentedControl.h"
#import "MBProgressHUD.h"
#import "ActionSheetAutoDismiss.h"

// Se mostrarán 4 seciones de 1 fila cada una
#define kNumSecciones               4

// Altura en píxeles de las diferentes celdas
#define kAltoZona                   54
#define kAltoFecha                  54
#define kAltoLCD                    148
#define kAltoAlarma                 54

// Imagen de fondo
#define kImagenFondoTable   @"gradientBackgroundClaro"

// Índices de los segmentos (debe ir en consonancia con los índices de
// "ZonaAzul.plist"
#define kZonaSegmentedIndexAzul     0
#define kZonaSegmentedIndexNaranja  1
#define kZonaSegmentedIndexRoja     2

// Valores por defecto que se cargan con viewWillAppear
#define kLCDHorasDefault            1
#define kLCDMinDefault              0

// Cantidad de píxeles que se empequeñecen los LCD
#define kMuevePixeles               2
// Duración de la animación agitar
#define kMueveDuracion              0.5
// Duración de la animación que intercamiba los LCD
#define kIntercambiaDuracion        0.25

// Tiempo mínimo que se muestra el HUD
#define kMinTiempoHUD               2.5
// Tamaño de las fuentes del HUD
#define kTamanoFuenteHUD            25
#define kTamanoFuenteDetalleHUD     20

/***************** Tags para acceso con subviewWithTag *****************/
// Cell Zona
#define kZonaLabelTag               11
#define kZonaSegmentedTag           12

// Cell Fecha
#define kFechaTituloTag             21
#define kFechaHoraTag               22

// Cell LCD
#define kLCDTiempoViewTag           31
#define kLCDHorasTag                310
#define kLCDUnidadHorasTag          311
#define kLCDMinutosTag              312
#define kLCDUnidadMinutosTag        313
#define kLCDTiempoImagenTag         314
#define kLCDEurosViewTag            32
#define kLCDEurosLabelTag           320
#define kLCDEurosSimboloTag         321
#define kLCDEurosImagenTag          322

// Cell Alarma
#define kAlarmaTituloTag            41
#define kAlarmaFechaTag             42
#define kAlarmaBotonTag             43

/***********************************************************************/


@interface CalcularTableViewController : UITableViewController <UIActionSheetDelegate , MBProgressHUDDelegate> {
    // Las 4 cells estáticas que provienen del nib.
    UITableViewCell *_cellZona;
    UITableViewCell *_cellFecha;
    UITableViewCell *_cellLCD;
    UITableViewCell *_cellAlarma;
    
    // Índice de cada una de las secciones, no son constantes para poder reposicionar
    // las secciones
    int kSeccionFecha;
    int kSeccionZona;
    int kSeccionLCD;
    int kSeccionAlarma;
    
    // Calculadora que engloba una Zona, un Periodo,
    // unos Euros y una Fecha de inicio.
    Calculadora *_calc;
    
    // HUD para las notificaciones de la alarma
    MBProgressHUD *pHUD;
}

@property (nonatomic, retain) IBOutlet UITableViewCell *cellZona;
@property (nonatomic, retain) IBOutlet UITableViewCell *cellFecha;
@property (nonatomic, retain) IBOutlet UITableViewCell *cellLCD;
@property (nonatomic, retain) IBOutlet UITableViewCell *cellAlarma;

@property (nonatomic, retain) Calculadora *calc;



/** Métodos que son llamados por accines de la interfaz **/
// Método que se llama cuando el usuario cambia la zona mediante el UISegmentedControl
- (IBAction) cambiaZona:(id)sender;
// Método que se llama al tocar el LCD que muestra el tiempo
- (IBAction) tocaLCDTiempo:(id)sender;
// Método que se llama al tocar el LCD que muestra los euros
- (IBAction) tocaLCDEuros:(id)sender;
// Método que se llama al tocar el botón de Alarma (tocando la cell de alarma también 
// se llamará a este método)
- (IBAction) ponerAlarma:(id)sender;

/** Métodos auxiliares de animación **/
// Devuelve verdadero si el view no está arriba (y=0)
- (BOOL) estaAbajo: (UIView *) v;
// Método que intercambia la posición de los LCDs de manera animada
- (void) interCambiaLCDs;
// Método que hace oscilar horizontalmente una view en un intervalo de tiempo con 
// una amplitud medida en píxeles.
// Se utiliza para mostrar que esa vista ha cambiado (ha sido editada)
- (void) agitaView: (UIView *) v conDuracion: (NSTimeInterval) dur amplitud: (int) a;
// Método que hace desaparecer y aparecer una view en una duración determinada
// Se utiliza para mostrar que se ha calculado el resultado en dicha vista.
- (void) resaltaView: (UIView *) v conDuracion: (NSTimeInterval) dur;

/** Métodos "Cambiadores" **/
- (void) actualizaZona: (Zona *) z animado: (BOOL) animado;
- (void) actualizaFecha: (Fecha *) f animado: (BOOL) animado;
- (void) actualizaPeriodo: (Periodo *) p animado: (BOOL) animado;
- (void) actualizaEuros: (Euros *) e animado: (BOOL) animado;

// Con los datos actuales en calc, teniendo en cuenta que LCD está arriba y cual abajo
// calcula los valores de los LCDs y la hora de fin
- (void) calculaResultadoAnimado: (BOOL) animado;

// Método utilizado para probar que los resultados de los 
// cálculos son correctos
- (void) test;
@end
