//
//  EurosActionSheetDelegate.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "EurosActionSheet.h"
//#define _DEBUG

@implementation EurosActionSheet
@synthesize picker = _picker;
@synthesize maxEuros = _maxEuros;

// Constructor con un sólo botón el delegate especificado
- (id)initWithButton:(NSString *) buttonTitle delegate: (id<UIActionSheetDelegate>) deleg euros: (Euros *) eur maxEuros: (Euros *) mEuros {
    // Llamar al constructor del padre
	self = [super initWithTitle: nil 
                       delegate: deleg 
              cancelButtonTitle: buttonTitle 
		 destructiveButtonTitle: nil 
              otherButtonTitles: nil];
	if (self) {
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(appWillResignActive) name:UIApplicationWillResignActiveNotification
                                                   object:[UIApplication sharedApplication]];
        
        // Poner la variable de instancia
        self.maxEuros = mEuros;
        
        // Instanciar un picker
		self.picker = [[UIPickerView alloc] initWithFrame:CGRectZero];

        [self.picker setDelegate: self];
        [self.picker setDataSource: self];
        
        // Para que muestre el indicador translúcido
        self.picker.showsSelectionIndicator = YES;
        
        // Ai los euros que nos dan son mayores que el máximo...
        if ([eur esMayorOIgual: self.maxEuros]) { 
            eur.euros = self.maxEuros.euros;
            eur.centimos = self.maxEuros.centimos; 
            [self.picker reloadComponent:kComponenteCentimos];
        }
        eurActual = eur.euros;
        centActual = eur.centimos;
        
        // Añadimos el picker como subvista
        [self addSubview: self.picker];
        
        layoutDone = false;
	}
	return self;
}

// Método que será llamado cuando la aplicación vaya a dejar de estar activa
- (void) appWillResignActive {
    // Hacemos dismiss a sí mismo
    [self dismissWithClickedButtonIndex:0 animated:NO];
}

- (void)dealloc {
    [_picker release];
    [_maxEuros release];
    [super dealloc];
}

#pragma mark - Métodos de dibujado

// Devuelve la coordenada Y más alta de todas las subvistas (que no sean el picker)
- (CGFloat) maxSubviewY {
	CGFloat maxY = 0;
    // Para todas las vistas en el array de subvistas
	for( UIView *view in self.subviews ){
		if(![view isKindOfClass:[UIPickerView class]]) { //Si no es el picker
			CGRect fr = [view frame];
			CGFloat actualY = fr.origin.y + fr.size.height;
			if(actualY > maxY)
				maxY = actualY;
		}
	}
	return maxY;
}

// Función que se llama al dibujar las vistas
- (void)layoutSubviews {
    // Lo primero, llamar al superior
	[super layoutSubviews];
	
    // Si no lo hemos hecho ya...
	if(!layoutDone) {
		CGRect fr = [self frame];
		CGFloat ancho = fr.size.width;
		CGFloat altoPicker = self.picker.frame.size.height;
        CGFloat maxY = [self maxSubviewY];
		
		// Meter UIPickerView abajo y el botón (solo habrá uno) 10 píxeles más arriba
		for(UIView *view in self.subviews){
		    if([view isKindOfClass:[UIPickerView class]]){
				[view setFrame: CGRectMake(0, maxY, ancho, altoPicker) ];
		    } else {
				CGRect frAux = [view frame];
				frAux.origin.y -= 10;
				[view setFrame:frAux];
			}
		}
		
		// Añadir el alto del picker al tamaño del actionsheet (+2)
		fr.size.height += altoPicker + 2.0;
        // Mover la coordenada y (altura) del actionsheet
		fr.origin.y -= altoPicker + 2.0;
		[self setFrame:fr]; 

        // Calcular la fila del céntimo a seleccionar
        int centASeleccionar;
        // Si hay 0 euros seleccionamos la fila - el offset (minCentimos)
        if (eurActual == 0)
            centASeleccionar = (centActual - kMinCentimos)/5;
        // Si es el máximo, calculamos normal
        else if (eurActual == self.maxEuros.euros)
            centASeleccionar = centActual / 5;
        // Si no es ni mínimo ni máximo, calculamos normal pero centrado dentro
        // de las kMAXROWS vueltas
        else
            centASeleccionar = (centActual / 5) + 20*((kMAXROWS/2)/20);
        
        
        [self.picker selectRow: eurActual inComponent: kComponenteEuros animated:YES];
        [self.picker selectRow: centASeleccionar inComponent: kComponenteCentimos animated:YES];
        
        
        // Añadimos los labels de las unidades
        UILabel *labelPunto = [[UILabel alloc] initWithFrame:CGRectMake(95, 85, kAnchoEuros, 45)];
        // Label Centrado
        labelPunto.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        labelPunto.opaque = NO;
        labelPunto.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        labelPunto.textColor = [UIColor darkGrayColor];
        // Poner sombra
        labelPunto.shadowColor = [UIColor whiteColor];
        labelPunto.shadowOffset = CGSizeMake(0, 1);
        // Negrita tamaño 15
        labelPunto.font = [UIFont boldSystemFontOfSize:20];
        labelPunto.text=@".";
        
        [self.picker addSubview: labelPunto];
        [labelPunto release];
        
        UILabel *labelEur = [[UILabel alloc] initWithFrame:CGRectMake(195, 85, kAnchoEuros, 45)];
        // Label Centrado
        labelEur.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        labelEur.opaque = NO;
        labelEur.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        labelEur.textColor = [UIColor darkGrayColor];
        // Poner sombra
        labelEur.shadowColor = [UIColor whiteColor];
        labelEur.shadowOffset = CGSizeMake(0, 1);
        // Negrita tamaño 15
        labelEur.font = [UIFont boldSystemFontOfSize:20];
        labelEur.text=@"€";
        
        [self.picker addSubview: labelEur];
        [labelEur release];

        
		layoutDone = YES;
	}
}

# pragma mark Picker Datasource

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    // 2 componentes, euros y céntimos
    return 2;
}


- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    NSInteger res = 0;
    switch (component) {
        case kComponenteEuros:
            // Máximo de euros+1 [0 , 1, ... maxEuros]
            res = self.maxEuros.euros + 1;
            break;
        case kComponenteCentimos:
            // Si los euros actuales son el máximo, hay menos filas
            if ( eurActual == self.maxEuros.euros ) {
                // El número de bloques de 5 céntimos + 1.
                // [0 , 5, ..., maxCentimos]
                res = (self.maxEuros.centimos / 5) + 1;
            }
            else if ( eurActual == 0 ) {
                // Si el euro actual es 0, hay que quitar de las 20 filas que harían
                // los céntimos, los bloques no seleccionables
                // Si minCentimos=15:  [15 , 20 , ... 95]
                res = 20 - kMinCentimos / 5;
            }
            else {
                res = kMAXROWS; // Circular
            }
            break;
        default:
            break;
    }
    return res;
}

- (CGFloat)pickerView:(UIPickerView *)pickerView widthForComponent:(NSInteger)component {
    NSInteger res = 0;
    switch (component) {
        case kComponenteEuros:
            res = kAnchoEuros;
            break;
        case kComponenteCentimos:
            res = kAnchoCentimos;
            break;
        default:
            break;
    }
    return res;
}

- (UIView *)pickerView:(UIPickerView *)pickerView viewForRow:(NSInteger)row forComponent:(NSInteger)component reusingView:(UIView *)view {
    
    
    UILabel *label = (UILabel *) view;
    if ( label == nil || ![label isMemberOfClass: [UILabel class]] ) {
        #ifdef _DEBUG
                NSLog(@"__%@__nil   || %d:%d" , NSStringFromSelector( _cmd) , component , row);
        #endif
        // Le ponemos kAnchoFecha a todos ya que es el más grande
        label = [[[UILabel alloc] initWithFrame:CGRectMake(0, 0, kAnchoCentimos, 45)] autorelease];
        
        // Label Centrado
        label.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        label.opaque = NO;
        label.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        label.textColor = [UIColor blackColor];
        // Negrita tamaño 20
        label.font = [UIFont boldSystemFontOfSize:20];
    }
    else {
        #ifdef _DEBUG
                NSLog(@"__%@__nonil   || %d:%d" , NSStringFromSelector( _cmd) , component , row);
        #endif
    }
    
    
    NSString *res = nil;
    switch (component) {
        case kComponenteEuros:
            // Una cifra, el numero de la fila es el numero de euros
            res = [NSString stringWithFormat: @"%d" , row];
            break;
        case kComponenteCentimos:
            // Queremos que salgan los números con 2 carácteres rellenando con 0 desde la izquierda
            // row%20 nos da números de 0 a 19, multiplicado por 5 nos da números:
            // 00 , 05 , 10 , 15 , ... , 90 , 95
            
            row = row % 20;
            if ( eurActual == 0 )
                row += (kMinCentimos / 5);
            res = [NSString stringWithFormat:@"%02d" , row*5];
            break;
        default:
            break;
    }
    
    // Asignamos el texto a la etiqueta
    label.text=res;
    // La devolvemos para que se autolibere
    return label;
}

#pragma mark Picker delegate

- (void) pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component {
    // Si han cambiado los euros, hay que hacer que el componente de los
    // céntimos cambie
    if ( component == kComponenteEuros ) {
        eurActual = row;
        
        /**** MAX ****/
        if ( eurActual == self.maxEuros.euros ) {
            // Si el céntimo actual es mayor que el máximo, nos quedamos con el máximo
            if (centActual > self.maxEuros.centimos )
                centActual = self.maxEuros.centimos;
            // Recargar y seleccionar
            [self.picker reloadComponent: kComponenteCentimos];
            [self.picker selectRow: centActual/5 inComponent:kComponenteCentimos animated:NO];

        }
        else {
            // Recargar y seleccionar
            [self.picker reloadComponent:kComponenteCentimos];
            [self.picker selectRow: (centActual/5) + 20*((kMAXROWS/2)/20) inComponent:kComponenteCentimos animated:NO];
        }
        
        /**** MIN ****/
        if ( eurActual == 0 ) {
            // Si el céntimo actual está por debajo del mínimo, subirlo
            if ( centActual < kMinCentimos )
                centActual = kMinCentimos;
            
            // Recargar y seleccionar (con offset)
            [self.picker reloadComponent: kComponenteCentimos];
            [self.picker selectRow: (centActual-kMinCentimos)/5 inComponent:kComponenteCentimos animated:NO];
        }
    }
    // Si han cambiado los céntimos, guardárnoslos
    else {
        centActual = (row % 20) * 5;
        // Si el euro actual es el 0, hay que tener en cuenta el offset de mínimo de euros
        if ( eurActual == 0 )
            centActual += kMinCentimos;
    }
    
    //Le hacemos creer que hemos hecho dismiss para que actualize el valor.
    [[super delegate] actionSheet:self willDismissWithButtonIndex: -1];
}

#pragma mark - Acceder

// Accede a los euros reflejados en el picker
- (Euros *) euros {
    Euros *res = [[Euros alloc] initWithEuros: eurActual
                                     centimos: centActual
                  ];
    return [res autorelease];
}

@end
