//
//  Fecha.m
//  iPark
//
//  Created by Mikel Elorz on 30/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Fecha.h"


@implementation Fecha
@synthesize dia = _dia;
@synthesize horas = _horas;
@synthesize minutos = _minutos;

#pragma mark - Comparadores, Constructores, ...

// Constructor con la fecha de ahora
- (id)initWithAhora {
    self = [super init];
    if (self) {
        // Poner la fecha del dia a ahora
        self.dia = [TraductorHash traducir:@"Hoy"];
        NSDate *date = [NSDate date];
        NSCalendar *calendar = [NSCalendar currentCalendar];
        // Obtener las horas y los minutos como integers
        NSDateComponents *components = [calendar components:(kCFCalendarUnitHour | kCFCalendarUnitMinute) 
                                                   fromDate:date];
        
        self.horas = [components hour];
        self.minutos = [components minute];
    }
    return self;
}

// Constructor con una fecha determinada
- (id) initWithDia: (NSString *) d horas: (int) h minutos: (int) m {
    self = [super init];
    if (self) {
        // Importante, traducir el mensaje
        self.dia = [TraductorHash traducir:d];
        self.horas = h;
        self.minutos = m;
    }
    return self;
}

- (void)dealloc {
    [_dia release];
    [super dealloc];
}

// Método que compara si un objeto representa la misma fecha
- (BOOL) isEqual:(id)object {
    BOOL res = FALSE;
    // Comparar si el objeto es de esta misma clase (y de paso que no sea nil)
    if  ( [object isKindOfClass: [self class]] ) {
        Fecha *f = (Fecha *) object;
        // Será igual si los componentes son iguales
        res = ( [f.dia isEqual: self.dia] && 
                 f.horas == self.horas && 
                 f.minutos == self.minutos );
    }
    return res;
}

// Método que devuelve la representación del objeto como string
- (NSString *) description {
    return [NSString stringWithFormat: @"%@ %02d:%02d" , self.dia , self.horas , self.minutos];
}


#pragma mark - Otros métodos
// Devuelve la fecha en formato NSDate
- (NSDate *) date {
    NSDate *date = [NSDate date];
    NSCalendar *calendar = [NSCalendar currentCalendar];
    // Obtener las horas y los minutos como integers
    NSDateComponents *components = [calendar components:(kCFCalendarUnitYear | kCFCalendarUnitMonth | kCFCalendarUnitDay | kCFCalendarUnitHour | kCFCalendarUnitMinute) 
                                               fromDate:date];
    
    NSDateFormatter *tempFormatter = [[NSDateFormatter alloc]init];
    [tempFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    
    int dia = ([self.dia isEqualToString: [TraductorHash traducir:@"Hoy"]])
        ? [components day] 
        : [components day] + 1;
    
    NSString* message = [NSString stringWithFormat:@"%04d-%02d-%02d %02d:%02d:%02d", [components year], [components month], dia , self.horas, self.minutos, 0];

    
    NSDate *res = [tempFormatter dateFromString:message];
    
    [tempFormatter release];
    

    return res;
}

@end
