//
//  FechaActionSheetDelegate.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "FechaActionSheet.h"
//#define _DEBUG

@implementation FechaActionSheet
@synthesize picker;

// Constructor con un sólo botón y el delegate especificado
- (id)initWithButton:(NSString *) buttonTitle delegate: (id<UIActionSheetDelegate>) deleg {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    // Llamar al constructor del padre
	self = [super initWithTitle: nil 
                       delegate: deleg 
              cancelButtonTitle: buttonTitle 
		 destructiveButtonTitle: nil 
              otherButtonTitles: nil];
    
	if (self) {
        // Queremos ser notificados para escondernos cuando la aplicación se minimice
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(appWillResignActive) name:UIApplicationWillResignActiveNotification
                                                   object:[UIApplication sharedApplication]];
        
        // Instanciar un picker
		self.picker = [[UIPickerView alloc] initWithFrame:CGRectZero];
        [self.picker setDelegate: self];
        [self.picker setDataSource: self];
        
        // Para que muestre el indicador translúcido
        self.picker.showsSelectionIndicator = YES;
        
        // Añadimos el picker como subvista
        [self addSubview: self.picker];

        layoutDone = false;
	}
	return self;
}

// Método que será llamado cuando la aplicación vaya a dejar de estar activa
- (void) appWillResignActive {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    // Hacemos dismiss a sí mismo
    [self dismissWithClickedButtonIndex:0 animated:NO];
}

- (void)dealloc {
    [picker release];
    [super dealloc];
}

#pragma mark - Métodos de dibujado

// Devuelve la coordenada Y más alta de todas las subvistas (que no sean el picker)
- (CGFloat) maxSubviewY {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
	CGFloat maxY = 0;
    // Para todas las vistas en el array de subvistas
	for( UIView *view in self.subviews ){
		if(![view isKindOfClass:[UIPickerView class]]) { //Si no es el picker
			CGRect fr = [view frame];
			CGFloat actualY = fr.origin.y + fr.size.height;
			if(actualY > maxY)
				maxY = actualY;
		}
	}
	return maxY;
}

// Función que se llama al dibujar las vistas
- (void) layoutSubviews {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    // Lo primero, llamar al superior
	[super layoutSubviews];
	
    // Si no lo hemos hecho ya...
	if(!layoutDone) {
		CGRect fr = [self frame];
		CGFloat ancho = fr.size.width;
		CGFloat altoPicker = self.picker.frame.size.height;
        CGFloat maxY = [self maxSubviewY];
		
		// Meter UIPickerView abajo y el botón (solo habrá uno) 10 píxeles más arriba
		for(UIView *view in self.subviews){
		    if([view isKindOfClass:[UIPickerView class]]){
				[view setFrame: CGRectMake(0, maxY, ancho, altoPicker) ];
		    } else {
				CGRect frAux = [view frame];
				frAux.origin.y -= 10;
				[view setFrame:frAux];
			}
		}
		
		// Añadir el alto del picker al tamaño del actionsheet (+2)
		fr.size.height += altoPicker + 2.0;
        // Mover la coordenada y (altura) del actionsheet
		fr.origin.y -= altoPicker + 2.0;
		[self setFrame:fr];  
        
        
        // Poner el picker con la fecha y hora actual
        Fecha *f = [[Fecha alloc] initWithAhora];
        // Seleccionar "Hoy" en el primer componente
        [self.picker selectRow: 0  inComponent:kComponenteFecha animated:YES];
        int horaASeleccionar = f.horas + 24*((kMAXROWS/2)/24);
        int minutoASeleccionar = f.minutos + 60*((kMAXROWS/2)/60);
        [self.picker selectRow: horaASeleccionar inComponent:kComponenteHora animated:YES];
        [self.picker selectRow: minutoASeleccionar inComponent:kComponenteMinuto animated:YES];
        [f release];
        
        // Añadir el label que señala las horas y minutos
        UILabel *labelDosPuntos = [[UILabel alloc] initWithFrame:CGRectMake(183, 84, kAnchoHora, 45)];
        // Label Centrado
        labelDosPuntos.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        labelDosPuntos.opaque = NO;
        labelDosPuntos.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        labelDosPuntos.textColor = [UIColor darkGrayColor];
        // Añadir sombra
        labelDosPuntos.shadowColor = [UIColor whiteColor];
        labelDosPuntos.shadowOffset = CGSizeMake(0, 1);
        // Negrita tamaño 20
        labelDosPuntos.font = [UIFont boldSystemFontOfSize:20];
        labelDosPuntos.text=@":";
        
        [self.picker addSubview: labelDosPuntos];
        [labelDosPuntos release];
        
		layoutDone = YES;
	}
}

# pragma mark - Picker Datasource

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    // 3 componentes, dia, fecha y hora.
    return 3;
}

- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    #ifdef _DEBUG
        NSLog(@"__%@__comp=%d" , NSStringFromSelector( _cmd),component);
    #endif
    NSInteger res = 0;
    switch (component) {
        case kComponenteFecha:
            // Hoy y mañana
            res = 2;
            break;
        case kComponenteHora:
            // Circular
            res = kMAXROWS;
            break;
        case kComponenteMinuto:
            // Circular
            res = kMAXROWS;
            break;
        default:
            break;
    }
    return res;
}

- (CGFloat)pickerView:(UIPickerView *)pickerView widthForComponent:(NSInteger)component {
    #ifdef _DEBUG
        NSLog(@"__%@__%d" , NSStringFromSelector( _cmd) , component);
    #endif
    NSInteger res = 0;
    switch (component) {
        case kComponenteFecha:
            res = kAnchoFecha;
            break;
        case kComponenteHora:
            res = kAnchoHora;
            break;
        case kComponenteMinuto:
            res = kAnchoMinuto;
            break;
        default:
            break;
    }
    return res;
}

- (UIView *)pickerView:(UIPickerView *)pickerView viewForRow:(NSInteger)row forComponent:(NSInteger)component reusingView:(UIView *)view {
    
    // Ver si la vista a reutilizar está ya instanciada y es de la clase correcta
    UILabel *label = (UILabel *) view;
    if ( label == nil || ![label isMemberOfClass: [UILabel class]] ) {
        #ifdef _DEBUG
                NSLog(@"__%@__nil   || %d:%d" , NSStringFromSelector( _cmd) , component , row);
        #endif
        // Le ponemos kAnchoFecha a todos ya que es el más grande
        label = [[[UILabel alloc] initWithFrame:CGRectMake(0, 0, kAnchoFecha, 45)] autorelease];
        
        // Label Centrado
        label.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        label.opaque = NO;
        label.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        label.textColor = [UIColor blackColor];
        // Negrita tamaño 20
        label.font = [UIFont boldSystemFontOfSize:20];
    }
    else {
        #ifdef _DEBUG
            NSLog(@"__%@__nonil   || %d:%d" , NSStringFromSelector( _cmd) , component , row);
        #endif
    }
    
    
    NSString *res;
    switch (component) {
        case kComponenteFecha:
            switch (row) {
                case 0: //HOY
                    res = [TraductorHash traducir:@"Hoy"];
                    /* El caso especial de "hoy" queremos que salga en azul con reborde blanco */
                    // Color cogido de manera empírica parecido al del UIDatePicker
                    label.textColor = [UIColor colorWithRed: 0.004 green: 0.376 blue: 0.949 alpha:1];
                    // Sombra blanca
                    label.shadowColor = [UIColor whiteColor];
                    // 1 píxel por debajo del label
                    label.shadowOffset = CGSizeMake(0, 1);
                    break;
                case 1: //MAÑANA
                    res = [TraductorHash traducir:@"Mañana"];
                    break;
                default:
                    res = @"error";
                    break;
            }
            break;

        case kComponenteHora:
            // Queremos que salgan los números con 2 carácteres rellenando con 0 desde la izquierda
            res = [NSString stringWithFormat:@"%02d" , row%24];
            break;
        case kComponenteMinuto:
            res = [NSString stringWithFormat:@"%02d" , row%60];
            break;
        default:
            res = @"error";
            break;
    }

    // Asignamos el texto a la etiqueta
    label.text=res;
    // La devolvemos para que se autolibere
    return label;
}

#pragma mark Picker delegate

// Cometar este método para que el valor del label de la hora no se actualice.
- (void) pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    //Le hacemos creer que hemos hecho dismiss para que actualize el valor.
    [[super delegate] actionSheet:self willDismissWithButtonIndex:-1];
}

#pragma mark - Acceder

// Accede a la fecha reflejada en el picker
- (Fecha *) fecha {
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    NSString *dia = nil;
    if ([picker selectedRowInComponent:kComponenteFecha] == 0)
        dia = [TraductorHash traducir:@"Hoy"];
    else if ([picker selectedRowInComponent:kComponenteFecha] == 1)
        dia = [TraductorHash traducir:@"Mañana"];
    
    Fecha * res = [[Fecha alloc] 
                   initWithDia: dia 
                         horas:[picker selectedRowInComponent:kComponenteHora] % 24
                       minutos:[picker selectedRowInComponent:kComponenteMinuto] % 60
                   ];
    return [res autorelease];
}

@end
