//
//  InfoDetailViewController.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "InfoDetailViewController.h"
//#define _DEBUG

@implementation InfoDetailViewController
@synthesize scrollView = _scrollview;
@synthesize detalle = _detalle;
@synthesize imageView = _imageView;
@synthesize textView = _textView;
@synthesize viewContenido = _viewContenido;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
    }
    return self;
}
                           
- (void)dealloc {
    [_scrollView release];
    [_detalle release];
    [_imageView release];
    [_textView release];
    [_viewContenido release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

// Método que se encarga de poner todos los elementos
// del detalle en sus correspondientes vistas y con
// los tamaños adecuados para el zoom
- (void) mostrarDetalle {
    // Ponerle el título en la navigation bar
    self.title = self.detalle.titulo; 
    
    // Si es la primera vez que se llama, el zoom debe ser 1
    // en sucesivas veces, solo se llama a mostrar detalle si el
    // detalle ha cambiado, y queremos que el zoom vuelva a la posición
    // inicial.
    [self.scrollView setZoomScale: 1 animated: YES];
    
    // Si tenemos imagen válida
    UIImage *img;
    if ( (img = [UIImage imageNamed: self.detalle.imagen]) ) {
        self.imageView.image = img;
    }
    // Si no, quitamos la vista de la imagen ajustando la posición del texto
    else {
        CGRect f = self.textView.frame;
        f.origin.y = self.imageView.frame.origin.y;
        self.textView.frame = f;
        [self.imageView removeFromSuperview];
    }
    // Resetear el contentSize para calcularlo desde cero
    self.scrollView.contentSize = CGSizeZero;
    
    // Ponerle el texto al UITextView
    self.textView.text = self.detalle.texto;
    // Hacer el textView del tamaño de su contenido
    CGRect fTextView = self.textView.frame;
    fTextView.size = self.textView.contentSize;
    self.textView.frame = fTextView;
    // Obtener el punto de más abajo del textview
    CGFloat maxY = fTextView.origin.y + fTextView.size.height;
    
    // Hacer el viewContenido del tamaño necesario
    CGRect fViewContenido = self.viewContenido.frame;
    fViewContenido.size.height = maxY;
    self.viewContenido.frame = fViewContenido;
    
    // Hacer que el tamaño del contenido del scrollView 
    // sea el nuevo, después de añadir el texto
    self.scrollView.contentSize = self.viewContenido.frame.size;
}


- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Configurar los límites del zoom y el delegate
    self.scrollView.maximumZoomScale = kMaxZoom;
	self.scrollView.minimumZoomScale = kMinZoom;
	self.scrollView.delegate = self;
    
    // Añadimos los parámetros del detalle
    [self mostrarDetalle];
    
    // Detectar gestos, código sacado del ejemplo de apple TapToZoom
    UITapGestureRecognizer *singleTap = [[UITapGestureRecognizer alloc] initWithTarget: self 
                                                                                action: @selector(handleSingleTap:)];
    UITapGestureRecognizer *doubleTap = [[UITapGestureRecognizer alloc] initWithTarget: self 
                                                                                action: @selector(handleDoubleTap:)];
    UITapGestureRecognizer *twoFingerTap = [[UITapGestureRecognizer alloc] initWithTarget: self 
                                                                                   action: @selector(handleTwoFingerTap:)];
    
    [doubleTap setNumberOfTapsRequired: 2];
    [twoFingerTap setNumberOfTouchesRequired: 2];
    
    // Añadimos los gestos a la vista del contenido, la vista
    // de texto seguirá detectando sus propios gestos
    [self.viewContenido addGestureRecognizer: singleTap];
    [self.viewContenido addGestureRecognizer: doubleTap];
    [self.viewContenido addGestureRecognizer: twoFingerTap];
    [singleTap release];
    [doubleTap release];
    [twoFingerTap release];
    
    // Gesto de tres dedos hacia la derecha para volver hacia atrás
    UISwipeGestureRecognizer *swipeDerecha = [[UISwipeGestureRecognizer alloc] initWithTarget:self.navigationController action: @selector(popViewControllerAnimated:)];
    swipeDerecha.direction = UISwipeGestureRecognizerDirectionRight;
    swipeDerecha.numberOfTouchesRequired = 3;
    [self.view addGestureRecognizer: swipeDerecha];
    [swipeDerecha release];

}

- (void)viewDidUnload {
    [self setDetalle: nil];
    [self setScrollView: nil];
    [self setImageView:nil];
    [self setTextView:nil];
    [self setViewContenido:nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear: animated];
    // Que se nos notifique al volver a activarse
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(appDidBecomeActive) 
                                                 name: UIApplicationDidBecomeActiveNotification
                                               object: nil];
    
    // Actualizar el detalle si se ha cambiado de idioma
    Detalle *det = [[Detalle alloc] initWithIndice: self.detalle.indice];
    if ( ![det isEqual: self.detalle] ) {
        // Si ha cambiado, asignar el nuevo detalle y mostrarlo
        self.detalle = det;
        [self mostrarDetalle];
    }
    // Si no ha cambiado, nada.
    [det release];
}
- (void) appDidBecomeActive {
    [self viewWillAppear:NO];
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver: self];
}

#pragma mark - ScrollView Delegate

- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView {
	return self.viewContenido;
}

/* Código cogido del ejemplo de apple TapToZoom, para gestionar los taps de zoom, etc. */
#pragma mark - TapDetectingImageViewDelegate methods

- (void)handleSingleTap:(UIGestureRecognizer *)gestureRecognizer {
    // single tap does nothing for now
}

- (void)handleDoubleTap:(UIGestureRecognizer *)gestureRecognizer {
    // double tap zooms in
    //if ( self.scrollView.zoomScale != 1 )
    float newScale = [self.scrollView zoomScale] * kZoomStep;
    CGRect zoomRect = [self zoomRectForScale: newScale 
                                  withCenter: [gestureRecognizer locationInView: gestureRecognizer.view]];
    [self.scrollView zoomToRect: zoomRect animated:YES];
}

- (void)handleTwoFingerTap:(UIGestureRecognizer *)gestureRecognizer {
    // two-finger tap zooms out
    float newScale = [self.scrollView zoomScale] / kZoomStep;
    CGRect zoomRect = [self zoomRectForScale: newScale 
                                  withCenter: [gestureRecognizer locationInView:gestureRecognizer.view]];
    [self.scrollView zoomToRect: zoomRect animated:YES];
}

#pragma mark Utility methods

- (CGRect) zoomRectForScale:(float)scale withCenter:(CGPoint)center {
    
    CGRect zoomRect;
    
    // the zoom rect is in the content view's coordinates. 
    //    At a zoom scale of 1.0, it would be the size of the imageScrollView's bounds.
    //    As the zoom scale decreases, so more content is visible, the size of the rect grows.
    zoomRect.size.height = [self.scrollView frame].size.height / scale;
    zoomRect.size.width  = [self.scrollView frame].size.width  / scale;
    
    // choose an origin so as to get the right center.
    zoomRect.origin.x    = center.x - (zoomRect.size.width  / 2.0);
    zoomRect.origin.y    = center.y - (zoomRect.size.height / 2.0);
    
    return zoomRect;
}

@end
