//
//  InfoTableViewController.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "InfoTableViewController.h"
//#define _DEBUG2

@implementation InfoTableViewController
@synthesize tabla = _tabla;

- (id)initWithStyle:(UITableViewStyle)style {
    self = [super initWithStyle:style];
    if (self) {
    }
    return self;
}

- (void)dealloc {
    [_tabla release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Si no tenemos inicializada la tabla, es que estamos en
    // el nodo raíz.
    if (self.tabla == nil) {
        #ifdef _DEBUG2
            NSLog(@"__%@__" , NSStringFromSelector( _cmd));
        #endif
        Tabla *t = [[Tabla alloc] initWithIndice: kIndiceTablaInicial];
        self.tabla = t;
        [t release];
    }
    // El título de la navigation bar es el título de la tabla actual
    self.title = self.tabla.titulo;

    // Definir el alto de todas las celdas
    self.tableView.rowHeight = kAltoCelda;
    
    // Añadir la imagen de fondo
    UIImageView *iv = [[UIImageView alloc] initWithImage: [UIImage imageNamed: kImagenFondoTable]];
    self.tableView.backgroundView = iv;
    [iv release];
    
    // Con tres dedos hacia la derecha, volvemos hacia atrás
    UISwipeGestureRecognizer *swipeDerecha = [[UISwipeGestureRecognizer alloc] 
                                              initWithTarget: self.navigationController
                                                      action: @selector(popViewControllerAnimated:)];
    swipeDerecha.direction = UISwipeGestureRecognizerDirectionRight;
    swipeDerecha.numberOfTouchesRequired = 3;
    [self.view addGestureRecognizer: swipeDerecha];
    [swipeDerecha release];
    
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // Recargar los ajustes, ver si hay cambios en las preferencias
    [[Ajustes instancia] cambiosEnPreferendias];
    // Recargar los datos de la tabla (traducciones)
    [self.tableView reloadData];
    
    // Que se nos notifique al volver a activarse
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(appDidBecomeActive) 
                                                 name: UIApplicationDidBecomeActiveNotification
                                               object: nil];
    

    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    
    // Recargar la tabla por si el idioma ha cambiado
    Tabla *t = [[Tabla alloc] initWithIndice: self.tabla.indice];
    self.tabla = t;
    [t release];
    [self.tableView reloadData];
    
    //Poner el (nuevo) título a la navigation bar
    self.navigationItem.title = self.tabla.titulo;
}

- (void) appDidBecomeActive {
    [self viewWillAppear:NO];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver: self];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return [self.tabla numeroSecciones];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    Seccion *sec = [self.tabla seccionNumero: section];
    return [sec numeroFilas];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    NSString *indice = [self.tabla.listaSecciones objectAtIndex: section];
    Seccion *sec = [[[Seccion alloc] initWithIndice: indice] autorelease];
    #ifdef _DEBUG
        NSLog(@"__%@__%@ , %@" , NSStringFromSelector( _cmd), sec.tituloSeccion , sec.listaFilas);
    #endif
    // Si el título de la sección es vacío, no se muestra
    return sec.tituloSeccion;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"InfoTableViewControllerCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] 
                   initWithStyle: UITableViewCellStyleSubtitle 
                 reuseIdentifier: CellIdentifier] 
                autorelease];

        /* Definir los parámetros comunes a todas las celdas */
        // Añadir la imagen de fondo
        UIImageView *iv = [[UIImageView alloc] initWithImage: [UIImage imageNamed: kImagenCell]];
        cell.backgroundView = iv;
        [iv release];
        // Añadir la imagen de fondo para cuando se seleccione la cell
        UIImageView *ivSelec = [[UIImageView alloc] initWithImage: [UIImage imageNamed: kImagenCellSelec]];
        cell.selectedBackgroundView = ivSelec;
        [ivSelec release];
        
        // Hacer que las dos etiquetas sean transparentes
        cell.detailTextLabel.backgroundColor = 
              cell.textLabel.backgroundColor = [UIColor clearColor];
        // Definir el tamaño de la fuente
        cell.textLabel.font = [UIFont boldSystemFontOfSize: kTamFuente];
    }
    
    /* Configurar los parámetros particulares de cada celda */
    // Obtener la fila actual
    Seccion *sec = [self.tabla seccionNumero: indexPath.section];
    Fila *fil = [sec filaNumero: indexPath.row];

    // Si la fila es detalle:
    if ( [fil isKindOfClass:[Detalle class]] ) {
        Detalle *det = (Detalle *) fil;
        // si no tiene texto, no se navega hacia ella y no es
        // seleccionable.
        if ( [det.texto isEqualToString:@""] ) {
            cell.accessoryType = UITableViewCellAccessoryNone;
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
        }
        // si tiene texto, es seleccionable y navegable
        else {
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
            cell.selectionStyle = UITableViewCellSelectionStyleGray;
        }
    }
    // si no, es una tabla, es seleccionable y navegable
    else {
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        cell.selectionStyle = UITableViewCellSelectionStyleGray;
    }

    // Poner el título y subtítulo correspondientes
    cell.textLabel.text = fil.titulo;
    cell.detailTextLabel.text = fil.subtitulo;
    
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Obtenemos la fila seleccionada (row) de la sección actual (section)
    Seccion *sec = [self.tabla seccionNumero: indexPath.section];
    Fila *fila = [sec filaNumero: indexPath.row];
    #ifdef _DEBUG2
        NSLog(@"__%@__ %@\n%@ %@ " , NSStringFromSelector( _cmd) , indexPath , fil.titulo , fil.indice);
    #endif
    
    // Si la fila es de tipo detalle:
    if ( [fila isKindOfClass:[Detalle class]] ) {
        Detalle *det = (Detalle *) fila;
        // Comprobamos ssi la fila tiene texto
        if ( ![det.texto isEqualToString:@""] ) {
            // El siguiente controlador es de detalle
            InfoDetailViewController *detailVC = 
                [[InfoDetailViewController alloc]
                 initWithNibName: @"InfoDetailViewController"
                          bundle: nil];
            
            // Le pasamos el detalle al nuevo controlador
            detailVC.detalle = det;
            [self.navigationController pushViewController: detailVC animated:YES];
            [detailVC release];
        }
        // else
            // es un detalle sin texto, no se navega.
    }
    // Si no, es de tipo Tabla
    else if ( [fila isKindOfClass:[Tabla class]] ) {
        InfoTableViewController *tableVC = 
            [[InfoTableViewController alloc]
             initWithNibName: @"InfoTableViewController"
                      bundle: nil];
        tableVC.tabla = (Tabla *) fila;
        #ifdef _DEBUG2
            NSLog(@"_%@_%@_%@_" , tableVC.tabla.titulo, tableVC.tabla.indice , tableVC.tabla.listaSecciones);
        #endif
        [self.navigationController pushViewController: tableVC animated:YES];
        [tableVC release];
    }
}

@end
