//
//  SectoresViewController.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "SectoresViewController.h"
//#define _DEBUG8

@implementation SectoresViewController
@synthesize mapView = _mapView;
@synthesize curlView = _curlView;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc {
    [_mapView release];
    [_curlView release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}


#pragma mark - View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    // Crear el curlview (para subir y bajar el mapa) y configurarlo
    FDCurlViewControl *cvc = [[FDCurlViewControl alloc] init];
    self.curlView = cvc;
    [cvc release];
    [self.curlView setTargetView: self.mapView];
    [self.curlView setDelegate: self];
    
    // Ponerle el tint al customsegmentedColor
    CustomSegmentedControl *csc = (CustomSegmentedControl *) [self.view viewWithTag: kSegmentedTag];
    csc.tintColor = [UIColor darkGrayColor];
    
    // Ponerle la imágene adecuadas para la pareja de estados selected highlighted
    UIButton *botonCurl = (UIButton *) [self.view viewWithTag: kBotonCurlTag];
    [botonCurl setImage: [UIImage imageNamed:@"curlSelec"]
               forState:UIControlStateSelected | UIControlStateHighlighted];
    
    // Añadir los reconocedores de gestos:
    // * 3 dedos hacia abajo, bajar el mapa
    // * 3 dedos hacia arriba, subir el mapa
    // * 3 dedos hacia la izquierda, ciclar por el tipo de mapas
    // * 3 dedos hacia la derecha, ciclar por el tipo de mapas hacia el otro lado
    UISwipeGestureRecognizer *swipeAbajo = [[UISwipeGestureRecognizer alloc] initWithTarget: self.curlView 
                                                                                     action: @selector(curlViewDown)];
    swipeAbajo.direction = UISwipeGestureRecognizerDirectionDown;
    swipeAbajo.numberOfTouchesRequired = 3;
    [self.view addGestureRecognizer: swipeAbajo];
    [swipeAbajo release];
    
    UISwipeGestureRecognizer *swipeArriba = [[UISwipeGestureRecognizer alloc] initWithTarget: self.curlView 
                                                                                      action: @selector(curlViewUp)];
    swipeArriba.direction = UISwipeGestureRecognizerDirectionUp;
    swipeArriba.numberOfTouchesRequired = 3;
    [self.mapView addGestureRecognizer: swipeArriba];
    [swipeArriba release]; 
    
    
    UISwipeGestureRecognizer *swipeIzquierda = [[UISwipeGestureRecognizer alloc] initWithTarget: self
                                                                                      action: @selector(ciclaTiposMapaIzquierda)];
    swipeIzquierda.direction = UISwipeGestureRecognizerDirectionLeft;
    swipeIzquierda.numberOfTouchesRequired = 3;
    [self.mapView addGestureRecognizer: swipeIzquierda];
    [swipeIzquierda release]; 
    
    
    UISwipeGestureRecognizer *swipeDerecha = [[UISwipeGestureRecognizer alloc] initWithTarget: self
                                                                                      action: @selector(ciclaTiposMapaDerecha)];
    swipeDerecha.direction = UISwipeGestureRecognizerDirectionRight;
    swipeDerecha.numberOfTouchesRequired = 3;
    [self.mapView addGestureRecognizer: swipeDerecha];
    [swipeDerecha release]; 
    
    // Cuando se cargue por primera vez, centrar el mapa
    [self irAlCentro: NO];
}

- (void)viewDidUnload {
    [self setMapView:nil];
    [self setCurlView: nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void) viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [[Ajustes instancia] cambiosEnPreferendias];
    // Quitamos todos los overlays que hubiere
    [self.mapView removeOverlays: self.mapView.overlays];
    // Quitaremos todas las anotaciones menos la user location
    NSMutableArray *oldAnnotations = [NSMutableArray arrayWithArray: self.mapView.annotations];
    [oldAnnotations removeObject: self.mapView.userLocation];
    [self.mapView removeAnnotations: oldAnnotations];
    // Añadir los polígonos y anotaciones cada vez que la vista aparezca
    // por si la aplicación ha cambiado de idioma o hay cambios en los sectores
    for (Sector *sect in [ZonaAzul instancia].sectores) {
        MKPolygon *pol = [sect poligono];
        [self.mapView addOverlay: pol];
        
        AnnotationSector *annot = [[[AnnotationSector alloc] init] autorelease];
        annot.coordinate = pol.coordinate;
        NSString *formato = [TraductorHash traducir:@"Sector %d"];
        annot.title = [NSString stringWithFormat:formato, sect.numero];
        [self.mapView addAnnotation: annot];
    }
    
    // Si está seleccionada la user location, desseleccionarla
    MKUserLocation *ul = self.mapView.userLocation;
    ul.title = [TraductorHash traducir:@"Su localización"];
    [self.mapView deselectAnnotation:ul animated:NO];

    // Habilitar o deshabilitar el botón segun el ajuste:
    UIButton *botonLocalizar = (UIButton *) [self.view viewWithTag: kBotonoLocalizarTag];
    BOOL sePuedeLocalizar = [Ajustes instancia].sePuedeLocalizar;
    botonLocalizar.enabled = sePuedeLocalizar;
    botonLocalizar.selected = (sePuedeLocalizar && botonLocalizar.selected);
    // Si se puede localizar y se estaba mostrando, mostrar la localización
    // Si no se puede localizar o no se estaba mostrando, no mostrar
    [self.mapView setShowsUserLocation: (sePuedeLocalizar && self.mapView.showsUserLocation) ];

    // Cambiar los labels
    UILabel *labelCentrar = (UILabel *) [self.view viewWithTag: kLabelCentrarTag];
    labelCentrar.text = [TraductorHash traducir:@"Centrar"];
    
    UILabel *labelLocalizar = (UILabel *) [self.view viewWithTag: kLabelLocalizarTag];
    labelLocalizar.text = [TraductorHash traducir:@"Localizar"];
    
    // Cambiar los titulos del segmented
    CustomSegmentedControl *csc = (CustomSegmentedControl *) [self.view viewWithTag: kSegmentedTag];
    [csc setTitle: [TraductorHash traducir:@"Mapa"] forSegmentAtIndex: kSegmentoMapa];
    [csc setTitle: [TraductorHash traducir:@"Satélite"] forSegmentAtIndex: kSegmentoSatelite];
    [csc setTitle: [TraductorHash traducir:@"Híbrido"] forSegmentAtIndex: kSegmentoHibrido];

    
    // No centramos el mapa al aparecer la vista
    //[self irAlCentro: NO];
    
    // Queremos que se nos notifique cuando la aplicación vuelve a estado activo para
    // resetear la interfaz (llamando a viewWillAppear)
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(appDidBecomeActive) 
                                                 name: UIApplicationDidBecomeActiveNotification
                                               object: nil];
    // Que se nos notifique cuando perdemos el foco para esconder el HUD
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(appWillResignActive) 
                                                 name: UIApplicationWillResignActiveNotification
                                               object: nil];
}

// Función que será llamada cuando la aplicación vuelva al estado activo
// estando la pestaña calcular mostrada.
- (void) appDidBecomeActive {
    [self viewWillAppear:NO];
}

- (void) appWillResignActive {
    if (self.curlView.isTargetViewCurled) 
        [self conmutaCurl: nil];
}

- (void) viewWillDisappear:(BOOL)animated {
    [super viewWillAppear:animated];
    // Bajar el mapa si está subido
    if (self.curlView.isTargetViewCurled) {
        [self.curlView curlViewDown];
    }
    [[NSNotificationCenter defaultCenter] removeObserver: self];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void) ciclaTiposMapaDerecha {
    CustomSegmentedControl *csc = (CustomSegmentedControl *) [self.view viewWithTag: kSegmentedTag];
    csc.selectedSegmentIndex = (csc.selectedSegmentIndex + 1) % csc.numberOfSegments;
}
- (void) ciclaTiposMapaIzquierda {
    CustomSegmentedControl *csc = (CustomSegmentedControl *) [self.view viewWithTag: kSegmentedTag];
    csc.selectedSegmentIndex = (csc.selectedSegmentIndex + csc.numberOfSegments - 1) % csc.numberOfSegments;
}

# pragma mark - Agitar

// 3 métodos necesarios para detectar cuando se agite:
-(BOOL)canBecomeFirstResponder {
    return YES;
}
-(void)viewDidAppear:(BOOL)animated {
    [self becomeFirstResponder];
}
- (void)motionEnded:(UIEventSubtype)motion withEvent:(UIEvent *)event {
	if (event.type == UIEventSubtypeMotionShake) {
        // Se ha agitado, centrar el mapa
        if ([self.curlView isTargetViewCurled])
            [self botonCentrar];
        else
            [self irAlCentro: YES];
	}
}

#pragma mark - FDCurlView Delegate

// Cuando el mapa se suba se llama a esta función
- (void)curlViewControlDidCurlViewUp:(FDCurlViewControl *)control {
    UIButton *botonCurl = (UIButton *) [self.view viewWithTag: kBotonCurlTag];
    [botonCurl setSelected: YES];
}
// Cuando la vista se baje, hay que ver si el mapa debe centrarse
- (void)curlViewControlDidCurlViewDown:(FDCurlViewControl *)control {
    UIButton *botonCurl = (UIButton *) [self.view viewWithTag: kBotonCurlTag];
    [botonCurl setSelected: NO];
    if (debeCentrarse) {
        [self irAlCentro: YES];
        debeCentrarse = NO;
    }
}

#pragma mark - Touches

// Si el mapa está subido, con toques en la parte superior de la pantalla
// hay que bajar el mapa
- (void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event {
    if (self.curlView.isTargetViewCurled) {
        for (UITouch *touch in touches) {
            CGPoint punto = [touch locationInView:self.view];
            // Vamos a ver si el toque ha sido dentro del mapa
            // para ello vamos a crear una recta y todo lo que esté
            // por encima de la recta será detectado
            /*    ____
                 |----|
                 |---/<--recta mx+n=y
                 |--/ |   toca en x, calculamos y(calc)
                 |-/  |   si y(toque) < y(calc) -> ha tocado dentro
                 |/   |
                 |____|
             */
            // Calcular que altitud (y) tiene la recta para 
            // la x del toque
            float m = -0.35;
            float n = self.view.frame.origin.y + self.view.frame.size.height*0.70;
            float x = punto.x;
            float y = m*x+n;
            // Si el toque ha sido por debajo de y (dentro del area 
            // que queremos capturar:
            if (punto.y < y)
                [self conmutaCurl:nil];
        }
    }
}

#pragma mark - Map delegate

// Creamos y devolvemos la vista para cada polígono
- (MKOverlayView *)mapView:(MKMapView *)mapView viewForOverlay:(id )overlay {
    MKPolygon *pol = overlay;
    int numSect = [pol.title intValue];
    #ifdef _DEBUG5
        NSLog(@"__%@__%d %@" , NSStringFromSelector( _cmd), numSect, pol.title);
    #endif
    MKPolygonView* polView = [[[MKPolygonView alloc] initWithPolygon: pol] autorelease];
    Sector *sect = [[ZonaAzul instancia] sectorNumero: numSect];
    // El color que tenga el sector
    polView.fillColor = sect.color;
    // Línea de color y ancho predefinidos
    polView.strokeColor = [UIColor darkGrayColor];
    if ([[UIScreen mainScreen] respondsToSelector:@selector(scale)] == YES && [[UIScreen mainScreen] scale] == 2.00) 
        // Tenemos pantalla retina
        polView.lineWidth = kAnchoLineaOverlay;
    else
        // No tenemos pantalla retina, la mitad de ancho
        polView.lineWidth = kAnchoLineaOverlay / 2;
    return polView;
}
// Determina si un polígono overlay es visible en el mapa
- (BOOL) esVisible: (MKPolygon *) pol {
    // Para ver si es visible un poligono en el mapa, nos vamos a basar
    // en comprobar si el punto central de la intersección entre la parte
    // visible del mapa y el rectángulo que engloba el overlay cae dentro
    // del overlay
    MKMapRect interSect = MKMapRectIntersection(self.mapView.visibleMapRect, pol.boundingMapRect);
    CLLocationCoordinate2D mapCoordinate = MKCoordinateRegionForMapRect(interSect).center;
    MKMapPoint mapPoint = MKMapPointForCoordinate(mapCoordinate);
    MKPolygonView *polygonView = (MKPolygonView *) [self.mapView viewForOverlay:pol];
    CGPoint polygonViewPoint = [polygonView pointForMapPoint: mapPoint];
    return CGPathContainsPoint(polygonView.path, NULL, polygonViewPoint, NO);

}
// Cuando se mueva el mapa, recalculamos la posición de las anotaciones de los sectores
- (void) mapView:(MKMapView *)map regionDidChangeAnimated:(BOOL)animated {
    // Quitaremos todas las anotaciones menos la user location
    NSMutableArray *oldAnnotations = [NSMutableArray arrayWithArray: self.mapView.annotations];
    [oldAnnotations removeObject: self.mapView.userLocation];
    [self.mapView removeAnnotations: oldAnnotations];
    for (Sector *sect in [ZonaAzul instancia].sectores) {
        MKPolygon *pol = [sect poligono];
        if ( [self esVisible: pol] ) {
            MKMapRect interSect = MKMapRectIntersection(self.mapView.visibleMapRect, pol.boundingMapRect);
            AnnotationSector *annot = [[[AnnotationSector alloc] init] autorelease];
            annot.coordinate = MKCoordinateRegionForMapRect(interSect).center;
            NSString *formato = [TraductorHash traducir:@"Sector %d"];
            annot.title = [NSString stringWithFormat:formato, sect.numero];
            [self.mapView addAnnotation: annot];
        }
    }
    #ifdef _DEBUG5
        NSLog(@"__%@__%f %f %f %f" , NSStringFromSelector( _cmd), self.mapView.region.center.latitude,  self.mapView.region.center.longitude,  self.mapView.region.span.latitudeDelta,  self.mapView.region.span.longitudeDelta);
    #endif
}
// Función que devuelve la vista concreta para una anotación
// Hay anotaciones de sectores y también está la anotación de la posición del usuario
- (MKAnnotationView *) mapView:(MKMapView *)map viewForAnnotation:(id <MKAnnotation>)annotation {
    MKAnnotationView *res;
    // Si la anotación es la de la localización del usuario, devolvemos nil para
    // que se ponga el círculo azul por defecto.
    if( [annotation isKindOfClass: [MKUserLocation class]] ) {
        MKUserLocation *ul = (MKUserLocation *) annotation;
        ul.title = [TraductorHash traducir:@"Su localización"];
        for (Sector *sect in [ZonaAzul instancia].sectores) {
            MKPolygon *pol = [sect poligono];
            if ( MKMapRectContainsPoint(pol.boundingMapRect, MKMapPointForCoordinate(ul.coordinate)) ) {
                ul.title = pol.title;
                break;
            }
        }
        
        
        res = nil;
    }
    // Si es otra anotación, es una anotación de Sector.
    else {
        static NSString *AnnotationViewID = @"annotationViewID";
        
        AnnotationSectorView *annotationView =
        (AnnotationSectorView *)[self.mapView dequeueReusableAnnotationViewWithIdentifier:AnnotationViewID];
        if (annotationView == nil) {
            annotationView = [[[AnnotationSectorView alloc]
                               initWithAnnotation:annotation 
                                  reuseIdentifier:AnnotationViewID] 
                              autorelease];
        }
        
        annotationView.annotation = annotation;
        res = annotationView;
    }
    return res;
}
// Función que se llama cuando el usuario toca en la anotación de su posición
// Debe calcular si su posición se encuentra dentro de un sector o no.
- (void)mapView:(MKMapView *)mapView didSelectAnnotationView:(MKAnnotationView *)view {
    if ( [view.annotation isKindOfClass: [MKUserLocation class]] ) {
        // Si no cae en ningún sector, tendrá este subtítulo
        NSString *tituloSector = [TraductorHash traducir:@"No es zona de pago"];
        for (Sector *sect in [ZonaAzul instancia].sectores) {
            MKMapPoint mapPoint = MKMapPointForCoordinate(self.mapView.userLocation.coordinate);
            #ifdef _DEBUG8
                // Tests de correcto funcionamiento
                //mapPoint = MKMapPointForCoordinate(CLLocationCoordinate2DMake(42.8050, -1.6520)); // Sector 4
                //mapPoint = MKMapPointForCoordinate(CLLocationCoordinate2DMake(42.8074, -1.6417)); // Sector 3
                //mapPoint = MKMapPointForCoordinate(CLLocationCoordinate2DMake(42.8121, -1.6500)); // Sin zona azul
                //mapPoint = MKMapPointForCoordinate(CLLocationCoordinate2DMake(42.8151, -1.6500)); // Sector 2
                mapPoint = MKMapPointForCoordinate(CLLocationCoordinate2DMake(42.8181, -1.6500)); // Sector 1
                NSLog(@"didSelect");
            #endif
            MKPolygon *pol = [sect poligono];
            MKPolygonView *polygonView = (MKPolygonView *) [self mapView: self.mapView viewForOverlay: pol];
            CGPoint polygonViewPoint = [polygonView pointForMapPoint: mapPoint];
            if (CGPathContainsPoint(polygonView.path, NULL, polygonViewPoint, NO)) {
                tituloSector = [NSString stringWithFormat: [TraductorHash traducir:@"Sector %d"]
                                                                                    , [pol.title intValue]];
                break;
            }
        }
        // Quitarle el título anterior y poner el nuevo
        self.mapView.userLocation.subtitle = nil;
        self.mapView.userLocation.subtitle = tituloSector;
    }
}


#pragma mark - Acciones
// Sube o baja el mapa según su estado
- (IBAction)conmutaCurl:(id)sender {
    if (self.curlView.isTargetViewCurled)
        [self.curlView curlViewDown];
    else
        [self.curlView curlViewUp];
}
// Mueve el mapa hasta centrarlo en Pamplona
- (void) irAlCentro: (BOOL) animado {
    CLLocationCoordinate2D centro = CLLocationCoordinate2DMake(42.810589, -1.646056);
    MKCoordinateSpan      espacio =       MKCoordinateSpanMake( 0.031320,  0.033240);
    MKCoordinateRegion     region =     MKCoordinateRegionMake(  centro ,  espacio );
    [self.mapView setRegion: region animated: animado];
}
// Acción que se llama cuando el usuario pulsa el botón de centrar
-(IBAction) botonCentrar{
    [self.curlView curlViewDown];
    // Se centrará el mapa cuando el mapa baje
    debeCentrarse = YES;
    #ifdef _DEBUG
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
}
// Localiza o deja de localizar al usuario mediante GPS
- (IBAction)conmutaLocalizar:(id)sender {
    [self.curlView curlViewDown];
    UIButton *botonLocalizar = sender;
    [botonLocalizar setSelected: !botonLocalizar.selected];
    [self.mapView setShowsUserLocation: ![self.mapView showsUserLocation]];


    debeIrALocalizacion = YES;
}
// Cambia el tipo de mapa según lo elegido
- (IBAction)cambiaMapa:(id)sender {
    [self.curlView curlViewDown];
    UISegmentedControl *sc = sender;
    switch ([sc selectedSegmentIndex]) {
        case kSegmentoMapa:
            self.mapView.mapType = MKMapTypeStandard;
            break;
        case kSegmentoSatelite:
            self.mapView.mapType = MKMapTypeSatellite;
            break;
        case kSegmentoHibrido:
            self.mapView.mapType = MKMapTypeHybrid;
            break;
        default:
            break;
    }
}

#pragma mark - Localizacion

// Cuando encuentre la localización del usuario, si es justo después de haberle
// dado a localizar, centrar el mapa en su posición actual
- (void)mapView:(MKMapView *)mapView didUpdateUserLocation:(MKUserLocation *)userLocation {
    if (debeIrALocalizacion) {
        [self irALocalizacionUsuario: YES];
        debeIrALocalizacion = NO;
    }
}
// Mover el mapa hasta la region con el mismo zoom (span)
// pero con la localización del usuario como centro.
- (void) irALocalizacionUsuario: (BOOL) animado{
    MKCoordinateRegion reg = self.mapView.region;
    reg.center = self.mapView.userLocation.coordinate;
    [self.mapView setRegion: reg animated: animado];
}


@end
