//
//  ShadowedTableView.m
//  ShadowedTableView
//
//  Created by Matt Gallagher on 2009/08/21.
//  Copyright 2009 Matt Gallagher. All rights reserved.
//

#import "SombrasTableView.h"

#define ALTURA_SOMBRA 20.0
#define ALTURA_SOMBRA_INVERSA 20.0
#define PROPORCION_SOMBRA (ALTURA_SOMBRA_INVERSA / ALTURA_SOMBRA)

@implementation SombrasTableView


- (CAGradientLayer *)sombraInversa:(BOOL)inversa {
	CAGradientLayer *nuevaSombra = [[[CAGradientLayer alloc] init] autorelease];
	CGRect frame =
		CGRectMake(0, 0, self.frame.size.width,
			inversa ? ALTURA_SOMBRA_INVERSA : ALTURA_SOMBRA);
	nuevaSombra.frame = frame;
	CGColorRef colorOscuro =
		[UIColor colorWithRed:0.0 green:0.0 blue:0.0 alpha:
         inversa ? 0.2 : 0.5].CGColor;
	CGColorRef colorClaro =
		[self.backgroundColor colorWithAlphaComponent:0.0].CGColor;
	nuevaSombra.colors =
		[NSArray arrayWithObjects:
			(id)(inversa ? colorClaro : colorOscuro),
			(id)(inversa ? colorOscuro : colorClaro),
		nil];
	return nuevaSombra;
}


- (void)layoutSubviews {
	[super layoutSubviews];
	
	if (!sombraOrigen) {
		sombraOrigen = [self sombraInversa:NO];
		[self.layer insertSublayer:sombraOrigen atIndex:0];
	}
	else if (![[self.layer.sublayers objectAtIndex:0] isEqual:sombraOrigen]) {
		[self.layer insertSublayer:sombraOrigen atIndex:0];
	}
	
	[CATransaction begin];
	[CATransaction setValue:(id)kCFBooleanTrue forKey:kCATransactionDisableActions];

	CGRect frameSombraOrigen = sombraOrigen.frame;
	frameSombraOrigen.size.width = self.frame.size.width;
	frameSombraOrigen.origin.y = self.contentOffset.y;
	sombraOrigen.frame = frameSombraOrigen;
	
	[CATransaction commit];
	
	NSArray *indexPathVisibles = [self indexPathsForVisibleRows];
	if ([indexPathVisibles count] == 0) {
		[sombraArriba removeFromSuperlayer];
		[sombraArriba release];
		sombraArriba = nil;
		[sombraAbajo removeFromSuperlayer];
		[sombraAbajo release];
		sombraAbajo = nil;
		return;
	}
	
	NSIndexPath *primeraFila = [indexPathVisibles objectAtIndex:0];
	if ([primeraFila section] == 0 && [primeraFila row] == 0) {
		UIView *cell = [self cellForRowAtIndexPath:primeraFila];
		if (!sombraArriba) {
			sombraArriba = [[self sombraInversa:YES] retain];
			[cell.layer insertSublayer:sombraArriba atIndex:0];
		}
		else if ([cell.layer.sublayers indexOfObjectIdenticalTo:sombraArriba] != 0) {
			[cell.layer insertSublayer:sombraArriba atIndex:0];
		}

		CGRect frameSombra = sombraArriba.frame;
		frameSombra.size.width = cell.frame.size.width;
        
		frameSombra.origin.y = -ALTURA_SOMBRA_INVERSA - [self rectForHeaderInSection:0].size.height;
		sombraArriba.frame = frameSombra;
	}
	else {
		[sombraArriba removeFromSuperlayer];
		[sombraArriba release];
		sombraArriba = nil;
	}

	NSIndexPath *ultimaFila = [indexPathVisibles lastObject];
	if ([ultimaFila section] == [self numberOfSections] - 1 &&
		[ultimaFila row] == [self numberOfRowsInSection:[ultimaFila section]] - 1) {
		UIView *cell =
			[self cellForRowAtIndexPath:ultimaFila];
		if (!sombraAbajo) {
			sombraAbajo = [[self sombraInversa:NO] retain];
			[cell.layer insertSublayer:sombraAbajo atIndex:0];
		}
		else if ([cell.layer.sublayers indexOfObjectIdenticalTo:sombraAbajo] != 0) {
			[cell.layer insertSublayer:sombraAbajo atIndex:0];
		}

		CGRect frameSombra = sombraAbajo.frame;
		frameSombra.size.width = cell.frame.size.width;
		frameSombra.origin.y = cell.frame.size.height;
		sombraAbajo.frame = frameSombra;
	}
	else {
		[sombraAbajo removeFromSuperlayer];
		[sombraAbajo release];
		sombraAbajo = nil;
	}
}

- (void)dealloc {
	[sombraArriba release];
	[sombraAbajo release];

	[super dealloc];
}


@end
