//
//  TiempoActionSheetDelegate.m
//  iPark
//
//  Created by Mikel Elorz on 12/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "TiempoActionSheet.h"


@implementation TiempoActionSheet
@synthesize picker = _picker;
@synthesize maxTiempo = _maxTiempo;


// Constructor con un sólo botón el delegate especificado
- (id)initWithButton:(NSString *) buttonTitle delegate: (id<UIActionSheetDelegate>) deleg periodo: (Periodo *) per maxTiempo: (Periodo *) mTiempo {
    // Llamar al constructor del padre
	self = [super initWithTitle: nil 
                       delegate: deleg 
              cancelButtonTitle: buttonTitle 
		 destructiveButtonTitle: nil 
              otherButtonTitles: nil];
	if (self) {
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(appWillResignActive) name:UIApplicationWillResignActiveNotification
                                                   object:[UIApplication sharedApplication]];
        
        // Poner la variable de instancia
        self.maxTiempo = mTiempo;
        
        // Instanciar un picker
		self.picker = [[UIPickerView alloc] initWithFrame:CGRectZero];
        [self.picker setDelegate: self];
        [self.picker setDataSource: self];
        
        // Para que muestre el indicador translúcido
        self.picker.showsSelectionIndicator = YES;
        
        // Si el periodo que nos dan es mayor que el máximo
        if ( [per esMayorOIgual: self.maxTiempo] ) {
            per.horas = self.maxTiempo.horas;
            per.minutos = self.maxTiempo.minutos; 
        }
        // Lo recargamos siempre
        [self.picker reloadComponent: kComponenteMinutos];

        horaActual = per.horas;
        minActual = per.minutos;
                
        // Añadimos el picker como subvista
        [self addSubview:self.picker];
        
        layoutDone = false;
	}
	return self;
}

- (void) appWillResignActive {
    [self dismissWithClickedButtonIndex: 0 animated:NO];
}

- (void)dealloc {
    [_picker release];
    [_maxTiempo release];
    [super dealloc];
}

#pragma mark - Métodos de dibujado

// Devuelve la coordenada Y más alta de todas las subvistas (que no sean el picker)
- (CGFloat) maxSubviewY {
	CGFloat maxY = 0;
    // Para todas las vistas en el array de subvistas
	for( UIView *view in self.subviews ){
		if(![view isKindOfClass:[UIPickerView class]]) { //Si no es el picker o los label
			CGRect fr = [view frame];
			CGFloat actualY = fr.origin.y + fr.size.height;
			if(actualY > maxY)
				maxY = actualY;
		}
	}
	return maxY;
}

// Función que se llama al dibujar las vistas
- (void)layoutSubviews {
    // Lo primero, llamar al superior
	[super layoutSubviews];
	
    // Si no lo hemos hecho ya...
	if(!layoutDone) {
		CGRect fr = [self frame];
		CGFloat ancho = fr.size.width;
		CGFloat altoPicker = self.picker.frame.size.height;
        CGFloat maxY = [self maxSubviewY];
		
		// Meter UIPickerView abajo y el botón (solo habrá uno) 10 píxeles más arriba
		for(UIView *view in self.subviews){
		    if([view isKindOfClass:[UIPickerView class]]){
				[view setFrame: CGRectMake(0, maxY, ancho, altoPicker) ];
		    } else {
				CGRect frAux = [view frame];
				frAux.origin.y -= 10;
				[view setFrame:frAux];
			}
		}
		
		// Añadir el alto del picker al tamaño del actionsheet (+2)
		fr.size.height += altoPicker + 2.0;
        // Mover la coordenada y (altura) del actionsheet
		fr.origin.y -= altoPicker + 2.0;
		[self setFrame:fr];  
        
        // Calcular el minuto a seleccionar de manera similar a EurosActionSheet
        int minASeleccionar;
        if (horaActual == 0)
            minASeleccionar = minActual - kMinMinutos;
        else if (horaActual == self.maxTiempo.horas)
            minASeleccionar = minActual;
        else
            minASeleccionar = minActual + 60*((kMAXROWS/2)/60);
        
        [self.picker selectRow: horaActual inComponent: kComponenteHoras animated:YES];
        [self.picker selectRow: minASeleccionar  inComponent: kComponenteMinutos animated:YES];
        
        
        // Añadir unidades
        UILabel *labelH = [[UILabel alloc] initWithFrame:CGRectMake(90, 85, kAnchoHoras, 45)];
        // Label Centrado
        labelH.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        labelH.opaque = NO;
        labelH.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        labelH.textColor = [UIColor darkGrayColor];
        // Poner sombra
        labelH.shadowColor = [UIColor whiteColor];
        labelH.shadowOffset = CGSizeMake(0, 1);
        // Negrita tamaño 15
        labelH.font = [UIFont boldSystemFontOfSize:15];
        labelH.text=@"h";
        
        [self.picker addSubview: labelH];
        [labelH release];
        
        UILabel *labelMin = [[UILabel alloc] initWithFrame:CGRectMake(192, 85, kAnchoHoras, 45)];
        // Label Centrado
        labelMin.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        labelMin.opaque = NO;
        labelMin.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        labelMin.textColor = [UIColor darkGrayColor];
        // Poner sombra
        labelMin.shadowColor = [UIColor whiteColor];
        labelMin.shadowOffset = CGSizeMake(0, 1);
        
        // Negrita tamaño 15
        labelMin.font = [UIFont boldSystemFontOfSize:15];
        labelMin.text=@"min";
        
        [self.picker addSubview: labelMin];
        [labelMin release];
        
		layoutDone = YES;
	}
}

# pragma mark Picker Datasource

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    // 2 componentes, horas y minutos
    return 2;
}


- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    NSInteger res = 0;
    switch (component) {
        case kComponenteHoras:
            // Máximo de euros+1 [0 .. maxEuros]
            res = self.maxTiempo.horas + 1;
            break;
        case kComponenteMinutos:
            // Si los euros actuales son el máximo, hay menos filas
            if ( horaActual == self.maxTiempo.horas )
                // [0 , 1 , ... maxTiempo]
                res = self.maxTiempo.minutos + 1;
            else if ( horaActual == 0 )
                // Si la hora actual es 0, hay que quitar de las 60 filas que harían
                // los minutos, los no seleccionables
                // Si minMinutos=10:  [10 , 11 , ... 59]
                res = 60 - kMinMinutos;
            else
                res = kMAXROWS; // Circular
            break;
        default:
            break;
    }
    return res;
}


- (CGFloat)pickerView:(UIPickerView *)pickerView widthForComponent:(NSInteger)component {
    NSInteger res = 0;
    switch (component) {
        case kComponenteHoras:
            res = kAnchoHoras;
            break;
        case kComponenteMinutos:
            res = kAnchoMinutos;
            break;
        default:
            break;
    }
    return res;
}

- (UIView *)pickerView:(UIPickerView *)pickerView viewForRow:(NSInteger)row forComponent:(NSInteger)component reusingView:(UIView *)view
{
    UILabel *label = (UILabel *) view;
    if ( label == nil || ![label isMemberOfClass: [UILabel class]] ) {
        #ifdef _DEBUG
                NSLog(@"__%@__nil   || %d:%d" , NSStringFromSelector( _cmd) , component , row);
        #endif
        // Le ponemos kAnchoFecha a todos ya que es el más grande
        label = [[[UILabel alloc] initWithFrame:CGRectMake(0, 0, kAnchoHoras, 45)] autorelease];
        
        // Label Centrado
        label.textAlignment = UITextAlignmentCenter;
        // Fondo transparente
        label.opaque = NO;
        label.backgroundColor = [UIColor clearColor];
        // Color del texto negro
        label.textColor = [UIColor blackColor];
        // Negrita tamaño 20
        label.font = [UIFont boldSystemFontOfSize:20];
    }
    else {
        #ifdef _DEBUG
                NSLog(@"__%@__nonil   || %d:%d" , NSStringFromSelector( _cmd) , component , row);
        #endif
    }
    
    
    
    NSString *res = nil;
    switch (component) {
        case kComponenteHoras:
            // Una cifra, el numero de la fila es el numero de euros
            res = [NSString stringWithFormat: @"%d" , row];
            break;
        case kComponenteMinutos:
            // Queremos que salgan los números con 2 carácteres rellenando con 0 desde la izquierda
            // row%20 nos da números de 0 a 19, multiplicado por 5 nos da números:
            // 00 , 05 , 10 , 15 , ... , 90 , 95
            
            row = row % 60;
            if ( horaActual == 0 )
                row += kMinMinutos;
            res = [NSString stringWithFormat:@"%02d" , row ];
            break;
        default:
            break;
    }
    
    // Asignamos el texto a la etiqueta
    label.text=res;
    // La devolvemos para que se autolibere
    return label;
}

#pragma mark Picker delegate

- (void) pickerView:(UIPickerView *) pickerView didSelectRow:(NSInteger) row inComponent:(NSInteger)component {
    
    // Si han cambiado las horas, hay que hacer que el componente de los
    // minutos cambie
    if ( component == kComponenteHoras ) {
        horaActual = row;
        
        /**** MAX ****/
        if ( horaActual == self.maxTiempo.horas) {
            // Si nos pasamos del máximo nos quedamos con él
            if (minActual > self.maxTiempo.minutos )
                minActual = self.maxTiempo.minutos;
            // Recargar y seleccionar
            [self.picker selectRow: minActual inComponent: kComponenteMinutos animated: NO];
            [self.picker reloadComponent: kComponenteMinutos];
        }
        else {
            #ifdef _DEBUG2
                NSLog(@"__%d__" , minActual + 60*((kMAXROWS/2)/60));
            #endif
            // Recargar y seleccionar
            [self.picker reloadComponent: kComponenteMinutos];
            [self.picker selectRow: minActual + 60*((kMAXROWS/2)/60)  inComponent:kComponenteMinutos animated:NO];

        }
        
        /**** MIN ****/
        if ( horaActual == 0 ) {
            if ( minActual < kMinMinutos )
                minActual = kMinMinutos;
            
            [self.picker reloadComponent: kComponenteMinutos];
            [self.picker selectRow: minActual-kMinMinutos inComponent:kComponenteMinutos animated:NO];
        }
    }
    else {
        minActual = row % 60;
        // Si la hora actual es 0, hay que tener en cuenta el offset de minutos
        if ( horaActual == 0 ) 
            minActual += kMinMinutos;
    }
    
    //Le hacemos creer que hemos hecho dismiss para que actualize el valor.
    [[super delegate] actionSheet:self willDismissWithButtonIndex: -1];
}

#pragma mark - Acceder

// Accede al periodo reflejado en el picker
- (Periodo *) periodo {
    Periodo *res = [[Periodo alloc]
                    initWithHoras: horaActual
                    minutos: minActual
                    ];
    return [res autorelease];
}



@end
