//
//  Zona.h
//  iPark
//
//  Clase que representa una zona (azul, naranja, ...)
//  e implementa unos métodos para realizar los cálculos
//  de tiempo o dinero con las tarifas adecuadas
//
//  Created by Mikel Elorz on 09/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "Euros.h"
#import "Periodo.h"

// Los valores mínimos de céntimos y euros
#define kMinMinutos     10
#define kMinCentimos    25
// El incremento (tarifa),
#define kIncCentimos    5
// En zonas como la azul 5 céntimos -> 3 min
#define kIncMinutosAzul 3
// En la zona naranja a partir de las 2 horas
// 5 céntimos -> 10 min
#define kIncMinutosNar  10

// El límite de tiempo a partir del cual
// la zona naranja es más barata (se le restarán
// los 10 minutos y 25 céntimos mínimos)
#define kLimAzulHor     2
#define kLimAzulMin     00
#define kLimAzulEur     2
#define kLimAzulCent    10

@interface Zona : NSObject {
    NSString *_nombre;
    Periodo *_maxTiempo;
    Euros *_maxEuros;
}

// El nombre de la zona
@property (nonatomic , retain) NSString *nombre;
// Los máximos de tiempo y dinero
@property (nonatomic , retain) Periodo *maxTiempo;
@property (nonatomic , retain) Euros *maxEuros;

// Constructor con todos los atributos
- (id)initWithNombre: (NSString *) n maxTiempo: (Periodo *) mTiempo maxEuros: (Euros *) mEuros;
// Devuelve verdadero si self y el objeto representan la misma zona
- (BOOL) isEqual:(id)object;
// Devuelve la representación como string
- (NSString *) description;

// Método que calcula los euros a partir de un periodo dado
- (Euros *) calcularConPeriodo: (Periodo *) p;
// Método que calcula el periodo a partir de unos euros dados
- (Periodo *) calcularConEuros: (Euros *) e;



@end
