//
//  Zona.m
//  iPark
//
//  Created by Mikel Elorz on 09/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "Zona.h"

//#define _DEBUG


@implementation Zona
@synthesize nombre = _nombre;
@synthesize maxTiempo = _maxTiempo;
@synthesize maxEuros = _maxEuros;

#pragma mark - Constructores, dealloc, etc.

// Constructor con todos los atributos
- (id)initWithNombre: (NSString *) n maxTiempo: (Periodo *) mTiempo maxEuros: (Euros *) mEuros {
    self = [super init];
    
    if (self) {
        [self setNombre: n];
        [self setMaxTiempo: mTiempo];
        [self setMaxEuros: mEuros];

    }
    return self;
}

- (void)dealloc {
    [_nombre release];
    [_maxEuros release];
    [_maxTiempo release];
    [super dealloc];
}

// Devuelve verdadero si self y el objeto representan la misma zona
- (BOOL) isEqual:(id)object {
    BOOL res = FALSE;
    if  ( [object isKindOfClass: [self class]] ) {
        Zona *z = (Zona *) object;
        // Iguales si todos los componentes iguales
        res = ( [z.maxEuros isEqual: self.maxEuros] &&
                [z.maxTiempo isEqual: self.maxTiempo] &&
                [z.nombre isEqual: self.nombre]);
    }
    return res;
}

- (NSString *) description {
    return [NSString stringWithFormat: @"%@, %@ max€, %@ maxTiempo" , self.nombre , self.maxEuros , self.maxTiempo];
}

#pragma mark - Otros métodos

// Método que calcula los euros a partir de un periodo dado
- (Euros *) calcularConPeriodo: (Periodo *) p {
    #ifdef _DEBUG
        NSLog(@"__%@__Per=%@" , NSStringFromSelector( _cmd) , p);
    #endif
    Euros *res;
    // Vamos a trabajar en minutos
    int minutos = p.horas*60 + p.minutos;
    
    if ( minutos < kMinMinutos ) {
        return nil;
    }
    else {
        // XX minutos iniciales ==> YY céntimos
        res = [[Euros alloc] initWithEuros: 0 centimos: kMinCentimos];
        minutos -= kMinMinutos;
        
        #ifdef _DEBUG
                NSLog(@"__%@__res1 :%@" , NSStringFromSelector( _cmd) , res);
        #endif
        
        // El límite a partir del cual se cobra tarifa naranja
        int minutosLimiteAzul = kLimAzulHor*60 + kLimAzulMin - kMinMinutos;
        
        // Calculamos los centimos de la zona naranja si procede
        if ( minutos > minutosLimiteAzul) {
            // Estamos en zona naranja
            int diferencia = minutos - minutosLimiteAzul;
            float bloquesF = (float) diferencia / (float)kIncMinutosNar;
            // Redondeamos los minutos hacia arriba ya que necesitaremos bloques enteros
            // hasta alcanzar al menos la cantidad de minutos necesaria
            int bloquesNaranjas = (int) ceilf(bloquesF);
            int centimos = bloquesNaranjas * kIncCentimos;
            [res anadeCentimos: centimos];
            // Ya hemos quitado los céntimos de diferencia
            minutos = minutosLimiteAzul;
        }
        
        #ifdef _DEBUG
                NSLog(@"__%@__res1 :%@" , NSStringFromSelector( _cmd) , res);
        #endif
        
        // Cálculamos los centimos del periodo que vale como la zona azul
        float bloquesF = (float) minutos / (float) kIncMinutosAzul;
        int bloquesAzules = (int) ceilf(bloquesF);
        int centimos = bloquesAzules * kIncCentimos;
        [res anadeCentimos: centimos];
    }
    return [res autorelease];
}

- (Periodo *) calcularConEuros: (Euros *) e {
    Periodo *res;
    
    // Trabajamos en céntimo
    int centimos = e.euros*100 + e.centimos;
    
    if ( centimos < kMinCentimos ){
        return nil;
    }
    else {
        // XX minutos iniciales ==> YY céntimos
        res = [[Periodo alloc] initWithHoras:0 minutos:kMinMinutos];
        centimos -= kMinCentimos;
        
        // El límite a partir del cual se cobra tarifa naranja
        int centimosLimiteAzul = kLimAzulEur*100 + kLimAzulCent - kMinCentimos;
        
        // Calcular los céntimos a precio naranja
        if ( centimos >= centimosLimiteAzul) {
            // Estamos en zona naranja
            int diferencia = centimos - centimosLimiteAzul;
            int bloquesNaranajas = diferencia / kIncCentimos;
            int minutos = bloquesNaranajas * kIncMinutosNar;
            // Le añadimos los minutos - 1 para redondear, ya que las máquinas
            // lo hacen de esta manera.
            [res anadeMinutos: minutos-1];
            #ifdef _DEBUG
                        NSLog(@"__%@__%@" , NSStringFromSelector( _cmd),res);
            #endif
            // Ya hemos quitado los céntimos de diferencia
            centimos = centimosLimiteAzul;
        }
        
        // Calcular los céntimos a precio azul
        int bloquesAzules = centimos / kIncCentimos;
        int minutos = bloquesAzules * kIncMinutosAzul;
        [res anadeMinutos: minutos];
    }
    return [res autorelease];
}


@end
