//
//  ZonaAzul.m
//  iPark
//
//  Created by Mikel Elorz on 09/03/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "ZonaAzul.h"
//#define _DEBUG8

@implementation ZonaAzul
@synthesize sectores = _sectores;
@synthesize zonas = _zonas;

// La instancia es estática, sólo tendremos una (singleton)
static ZonaAzul *_instancia;

// Init que será llamado cuando accedamos a la instancia y no esté inicializada
- (id) init {
    self = [super init];
    if (self) {
        // Leemos los dos arrays del plist (estarán como dos entradas del diccionario)
        NSString *pathPlist = [[NSBundle mainBundle] pathForResource: kNombrePlist ofType:@"plist"];
        NSDictionary *dic = [NSDictionary dictionaryWithContentsOfFile:pathPlist];
        
        // Se lo quedan ya que son @property (nonatomic , retain )
        self.zonas = [NSArray arrayWithArray: [dic objectForKey:kKeyZonas]];
        // Cargaremos los sectores solo si es necesario, lazy loading
        //self.sectores = [self iniciaSectores];
        self.sectores = nil;
    }
    return self;
}

// Método de clase que devuelve la instacia de la Zona Azul y la crea de ser necesario
+ (ZonaAzul *) instancia {
    // Si la instancia existe devolverla directamente
    if(_instancia)
        return _instancia;
    
    // Por cuestiones de rendimiento, solo se sincroniza la parte de crear el objeto
    // (para que no haya 2 instancias)
    @synchronized( self ) {
        _instancia = [[self alloc] init];
    }
    return _instancia;
}

- (void)dealloc {
    [_sectores release];
    [_zonas release];
    [super dealloc];
}


# pragma mark - zona;

// Devuelve la zona con la posición i del array de zonas
- (Zona *) zonaConIndice: (int) i {
    // En el array hemos guardado el diccionario sin procesar
    // por no guardar todas las zonas, euros, etc. instanciados
    // (lazy loading)
    NSDictionary *dicZona = [self.zonas objectAtIndex: i];

    // Obtenemos el periodo máximo para la zona
    Periodo *maxT = [[Periodo alloc] initWithHoras: [[dicZona objectForKey: kKeyZonaMaxHoras] intValue]
                                           minutos: [[dicZona objectForKey: kKeyZonaMaxMinutos] intValue]
                     ];
    // Obtenemos la cantidad máxima de la zona
    Euros *maxE = [[Euros alloc] initWithEuros: [[dicZona objectForKey: kKeyZonaMaxEuros] intValue]
                                      centimos: [[dicZona objectForKey: kKeyZonaMaxCentimos] intValue]
                   ];
    
    // Instanciamos la zona con los máximos y el nombre
    Zona *res = [[Zona alloc] 
                     initWithNombre: [dicZona objectForKey:kKeyZonaNombre]
                          maxTiempo: maxT
                           maxEuros: maxE ];
    [maxT release];
    [maxE release];
    return [res autorelease];
}

# pragma mark - Sectores

// Método que devuelve el contenido del fichero KML online o de la copia local
- (NSString *) contenidoFicheroKML {
    NSString *res;

    NSError *e = [[[NSError alloc] init] autorelease];
    NSURL *url = [NSURL URLWithString: kURLKMLOnline];
    #ifdef _DEBUG5
        NSLog(@"__%@__" , NSStringFromSelector( _cmd));
    #endif
    res = [NSString stringWithContentsOfURL:url encoding:NSUTF8StringEncoding error:&e];
    // Si no se ha podido leer, devuelve nil.
    if (res == nil) {
        // Si cogerlo online falla, cogemos la versión local
        NSString *pathKML = [[NSBundle mainBundle] pathForResource: kFicheroKMLOffline  ofType: kExtenKMLOffline];
        res = [NSString stringWithContentsOfFile:pathKML encoding:NSUTF8StringEncoding error: &e];
    }
    return res;
}

// Dado el contenido de un fichero kml, para un número de sector n devuelve una lista
// de coordenadas separadas por saltos de línea con componentes separados por comas
- (NSString *) listaCoordenadasParaSector: (int) n enFichero: (NSString *) contenido {
    NSString *res = @"";
    // Iniciamos el scanner con el fichero KML
    NSScanner *sc = [NSScanner scannerWithString:contenido];
    // Buscamos hasta encontrar el nombre del sector n
    NSString *marcaSector = [NSString stringWithFormat:@"<name>Sector %d</name>" , n];
    [sc scanUpToString: marcaSector intoString: nil];
    // Buscamos hasta encontrar el tag coordinates
    NSString *principioCoordenadas = @"<coordinates>";
    [sc scanUpToString: principioCoordenadas intoString: nil];
    // Avanzamos la longitud del tag
    [sc setScanLocation:[sc scanLocation] + [principioCoordenadas length]];
    // Leemos desde el principio hasta el final de las coordenadas en el string resultado
    NSString *finCoordenadas = @"</coordinates>";
    [sc scanUpToString: finCoordenadas intoString: &res];
    #ifdef _DEBUG2
        NSLog(@"__%@__%@" , NSStringFromSelector( _cmd), res);
    #endif
    return res;
}

// Dada la lista de coordenadas anterior, creamos un array de objetos del tipo Coordenada
- (NSArray *) arrayCoordenadasDesdeLista: (NSString *) lista {
    // Dividirlos por líneas, cada coordenada en una línea
    NSArray *arrayStrings = [lista componentsSeparatedByString:@"\n"];
    NSMutableArray *res = [NSMutableArray array];
    // Para cada string de coordenada
    for (NSString *st in arrayStrings) {
        // Quitar los espacios y tabuladores al comienzo
        st = [st stringByTrimmingCharactersInSet: [NSCharacterSet characterSetWithCharactersInString:@" \t"]];
        // Si la línea no es vacía:
        if ( [st length] > 0) {
            // Crear una nueva coordenada a partir del string
            Coordenada *coord = [[[Coordenada alloc] initWithString: st] autorelease];
            #ifdef _DEBUG
                NSLog(@"__coord lat: %f lon: %f__" , coord.latitud , coord.longitud);
            #endif
            // Añadirla al array
            [res addObject: coord];
        }
    }
    return res;
}

// Buscar y crear el color para el sector n
- (UIColor *) colorParaSector: (int) n enFichero: (NSString *) contenido {
    UIColor *res;
    
    // Primero debemos encontrar para el sector dado, cuál es su estilo asociado
    NSScanner *scEstilo = [NSScanner scannerWithString:contenido];
    // Escanear hasta el nombre del sector
    NSString *marcaSector = [NSString stringWithFormat:@"<name>Sector %d</name>" , n];
    [scEstilo scanUpToString: marcaSector intoString: nil];
    // Escanear hasta el tag de stilo
    NSString *marcaEstiloSector = @"<styleUrl>#";
    [scEstilo scanUpToString: marcaEstiloSector intoString: nil];
    // Avanzar la longitud del tag de apertura
    [scEstilo setScanLocation: [scEstilo scanLocation] + [marcaEstiloSector length]];
    
    // Leer hasta el fin del tag
    NSString *marcaFinEstiloSector = @"</styleUrl>";
    NSString *estilo;
    [scEstilo scanUpToString: marcaFinEstiloSector intoString: &estilo];
    #ifdef _DEBUG3
        NSLog(@"__%@__ %@" , NSStringFromSelector( _cmd), estilo);
    #endif
    
    // Sabiendo el estilo, ahora debemos buscar el color con un nuevo escanner
    NSScanner *scColor = [NSScanner scannerWithString: contenido];
    // Buscar el estilo correspondiente al sector n
    NSString *marcaComienzoEstilo = [NSString stringWithFormat: @"<Style id=\"%@\">" , estilo];
    [scColor scanUpToString: marcaComienzoEstilo intoString:nil];
    // Avanzar hasta el tag polystyle
    NSString *marcaPolyStile = @"<PolyStyle>";
    [scColor scanUpToString: marcaPolyStile intoString:nil];
    // Avanzar hasta el tag del color
    NSString *marcaColor = @"<color>";
    [scColor scanUpToString: marcaColor intoString:nil];
    // Avanzar la longitud del tag de apertura
    [scColor setScanLocation: [scColor scanLocation] + [marcaColor length]];
    
    // El color está con el formato: 'valor hexBinary: aabbggrr'
    // ( https://code.google.com/intl/es/apis/kml/documentation/kmlreference.html )
    unsigned int colorHex;
    [scColor scanHexInt: &colorHex];
    #ifdef _DEBUG3
        NSLog(@"__%@__ %x" , NSStringFromSelector( _cmd), colorHex);
    #endif
    // Ir rotando y quedandonos con los 8 bits más a la derecha
    unsigned int red    = colorHex & 0x000000FF;
    colorHex >>= 8;
    unsigned int green  = colorHex & 0x000000FF;
    colorHex >>= 8;
    unsigned int blue   = colorHex & 0x000000FF;
    colorHex >>= 8;
    unsigned int alpha  = colorHex & 0x000000FF;
    
    #ifdef _DEBUG3
        NSLog(@"__%@__%x %x %x %x" , NSStringFromSelector( _cmd), red, green , blue, alpha);
    #endif
    // El color está de 0 a 255, UIColor necesita el porcentaje...
    res = [UIColor colorWithRed: (float) (red)/255 
                          green: (float) (green)/255
                           blue: (float) (blue)/255 
                          alpha: (float) (alpha)/255];
    return res;
}

- (Sector *) sectorNumero: (int) n {
    // En este punto el array está inicializado
    // El array está 0-indexado, el sector n está en la posición n-1;    
    return [self.sectores objectAtIndex: n-1];
}

- (int) numSectores {
    return [self.sectores count];
}

- (void) iniciaSectores {
    // Calcular el número de sectores que aparecen en el fichero
    NSString *contenido = [self contenidoFicheroKML];
    // Para ello vamos a contar el número de veces que aparece placemark:
    NSString *aEncontrar = @"<Placemark>";
    // Vemos cuantos caracteres de diferencia hay al sustituir <Placemark> por vacío.
    int diferencia = [contenido length] - [[contenido stringByReplacingOccurrencesOfString: aEncontrar 
                                                                                withString: @""] length];
    // Tantos caracteres dividido entre la longitud de 
    // la cadena a encontrar nos da el número de ocurrencias
    int numSectores = diferencia / [aEncontrar length];
    #ifdef _DEBUG8
        NSLog(@"__%@__ %d" , NSStringFromSelector( _cmd) , numSectores);
    #endif
    
    // Crear array temporal de sectores
    NSMutableArray *arrayTemp = [NSMutableArray arrayWithCapacity: numSectores];
    for (int i=0; i<numSectores; ++i) {
        NSArray *coord = [self arrayCoordenadasDesdeLista: 
                 [self listaCoordenadasParaSector: i+1 enFichero: contenido]];
        
        UIColor *color = [self colorParaSector: i+1 enFichero: contenido];
        
        Sector *sect = [[Sector alloc] initWithNumero: i+1 
                                             color: color 
                                       coordenadas: coord];
        
        [arrayTemp addObject: sect];
        [sect release];
    }
    self.sectores = arrayTemp;
}

// Debemos reescribir el getter para que lo inicialice cuando necesite (lazy loading)
- (NSArray *) sectores {
    // Inicializamos los sectores si no lo están ya.
    @synchronized( self ) {
        if (_sectores == nil) {
            [self iniciaSectores];
        }
    }
    return _sectores;
}
@end
